# -*- coding: utf-8 -*-
import copy
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Articulation(AbjadValueObject):
    r'''Articulation.

    ..  container:: example

        **Example 1.** Initializes from name:

        ::

            >>> Articulation('staccato')
            Articulation('staccato')

    ..  container:: example

        **Example 2.** Initializes from abbreviation:

        ::

            >>> Articulation('.')
            Articulation('.')

    ..  container:: example

        **Example 3.** Initializes from other articulation:

        ::

            >>> articulation = Articulation('staccato')
            >>> Articulation(articulation)
            Articulation('staccato')

    ..  container:: example

        **Example 4.** Initializes with direction:

        ::

            >>> Articulation('staccato', Up)
            Articulation('staccato', Up)

    .. container:: example

        **Example 5.** Use `attach()` to attach articulations to notes, rests
        or chords:

        ::

            >>> note = Note("c'4")
            >>> articulation = Articulation('staccato')
            >>> attach(articulation, note)
            >>> show(note) # doctest: +SKIP

    ..  todo:: Simplify initializer. Allow only initialization from name.
        Implement new ``from_abbreviation()`` and ``from_articulation()``
        methods to replace existing initializer polymorphism.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_direction',
        '_format_slot',
        '_name',
        )

    # this causes unnecessary coupling to changeable lilypond codebase
    # and is discouraged
    _articulations_supported = (
        'accent',
        'marcato',
        'staccatissimo',
        'espressivo'
        'staccato',
        'tenuto'
        'portato'
        'upbow',
        'downbow'
        'flageolet'
        'thumb',
        'lheel'
        'rheel'
        'ltoe',
        'rtoe'
        'open'
        'stopped',
        'turn'
        'reverseturn'
        'trill',
        'prall'
        'mordent'
        'prallprall'
        'prallmordent',
        'upprall',
        'downprall',
        'upmordent',
        'downmordent',
        'pralldown',
        'prallup',
        'lineprall',
        'signumcongruentiae',
        'shortfermata',
        'fermata',
        'longfermata',
        'verylongfermata',
        'segno',
        'coda',
        'varcoda',
        '^',
        '+',
        '-',
        '|',
        '>',
        '.',
        '_',
        )

    # this causes unnecessary coupling to changeable lilypond codebase
    # and is discouraged
    _shortcut_to_word = {
        '^': 'marcato',
        '+': 'stopped',
        '-': 'tenuto',
        '|': 'staccatissimo',
        '>': 'accent',
        '.': 'staccato',
        '_': 'portato',
        }

    ### INITIALIZER ###

    def __init__(self, name=None, direction=None):
        self._default_scope = None
        if isinstance(name, type(self)):
            argument = name
            name = argument.name
            direction = direction or argument.direction
        name = str(name)
        if '\\' in name:
            direction, name = name.split('\\')
            direction = direction.strip()
            name = name.strip()
        direction = \
            stringtools.to_tridirectional_ordinal_constant(direction)
        directions = (Up, Down, Center, None)
        assert direction in directions, repr(direction)
        self._name = name
        self._direction = direction
        self._format_slot = 'right'

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats articulation.

        Set `format_specification` to `''`, `'lilypond`' or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        elif format_specification == 'lilypond':
            return self._get_lilypond_format()
        return str(self)

    def __illustrate__(self):
        r'''Illustrates articulation.

        Returns LilyPond file.
        '''
        import abjad
        note = abjad.Note("c'4")
        articulation = copy.copy(self)
        abjad.attach(articulation, note)
        lilypond_file = abjad.LilyPondFile.new(note)
        lilypond_file.header_block.tagline = False
        return lilypond_file

    def __str__(self):
        r'''Gets string representation of articulation.

        Returns string.
        '''
        if self.name:
            string = self._shortcut_to_word.get(self.name)
            if not string:
                string = self.name
            if self.direction is None:
                direction = '-'
            else:
                direction = stringtools.to_tridirectional_lilypond_symbol(
                    self.direction)
            return '{}\{}'.format(direction, string)
        else:
            return ''

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        if self.direction is not None:
            return '{!r}, {!r}'.format(self.name, self.direction)
        else:
            return repr(self.name)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = [self.name]
        if self.direction is not None:
            values.append(self.direction)
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_is_indented=False,
            )

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.articulations.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of articulation.

        ..  container:: example

            >>> articulation = Articulation('staccato')
            >>> articulation.default_scope is None
            True

        Returns none.
        '''
        return self._default_scope

    @property
    def direction(self):
        r'''Gets direction of articulation.

        ..  container:: example

            **Example 1.** Without direction:

            >>> articulation = Articulation('staccato')
            >>> articulation.direction is None
            True

        ..  container:: example

            **Example 2.** With direction:

            >>> articulation = Articulation('staccato', direction=Up)
            >>> articulation.direction
            Up

        Returns ordinal constant or none.
        '''
        return self._direction

    @property
    def name(self):
        r'''Gets name of articulation.

        ..  container:: example

            **Example 1.** Staccato:

            ::

                >>> articulation = Articulation('staccato')
                >>> articulation.name
                'staccato'

        ..  container:: example

            **Example 2.** Tenuto:

            ::

                >>> articulation = Articulation('tenuto')
                >>> articulation.name
                'tenuto'

        Returns string.
        '''
        return self._name



# -*- coding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject


class MarkupCommand(AbjadValueObject):
    r'''LilyPond markup command.

    ..  container:: example

        Initializes a complex LilyPond markup command:

        ::

            >>> circle = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
            >>> square = markuptools.MarkupCommand('rounded-box', 'hello?')
            >>> line = markuptools.MarkupCommand('line', [square, 'wow!'])
            >>> rotate = markuptools.MarkupCommand('rotate', 60, line)
            >>> combine = markuptools.MarkupCommand('combine', rotate, circle)

        ::

            >>> print(format(combine, 'lilypond'))
            \combine
                \rotate
                    #60
                    \line
                        {
                            \rounded-box
                                hello?
                            wow!
                        }
                \draw-circle
                    #2.5
                    #0.1
                    ##f

        ::

            >>> note = Note("c'4")
            >>> markup = markuptools.Markup(combine)
            >>> attach(markup, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print(format(note))
            c'4
                - \markup {
                    \combine
                        \rotate
                            #60
                            \line
                                {
                                    \rounded-box
                                        hello?
                                    wow!
                                }
                        \draw-circle
                            #2.5
                            #0.1
                            ##f
                    }

    ..  container:: example

        Works with the LilyPond ``\score`` markup command:

        ::

            >>> small_staff = Staff("fs'16 gs'16 as'16 b'16")
            >>> small_staff.remove_commands.append('Clef_engraver')
            >>> small_staff.remove_commands.append('Time_signature_engraver')
            >>> set_(small_staff).font_size = -3
            >>> layout_block = lilypondfiletools.Block(name='layout')
            >>> layout_block.indent = 0
            >>> layout_block.ragged_right = True
            >>> command = markuptools.MarkupCommand(
            ...     'score',
            ...     [small_staff, layout_block],
            ...     )

        ::

            >>> f(command)
            \score
                {
                    \new Staff \with {
                        \remove Clef_engraver
                        \remove Time_signature_engraver
                        fontSize = #-3
                    } {
                        fs'16
                        gs'16
                        as'16
                        b'16
                    }
                    \layout {
                        indent = #0
                        ragged-right = ##t
                    }
                }

        ::

            >>> markup = Markup(contents=command, direction=Up)
            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> attach(markup, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'4
                    ^ \markup {
                        \score
                            {
                                \new Staff \with {
                                    \remove Clef_engraver
                                    \remove Time_signature_engraver
                                    fontSize = #-3
                                } {
                                    fs'16
                                    gs'16
                                    as'16
                                    b'16
                                }
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                        }
                d'4
                e'4
                f'4
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_args',
        '_command',
        '_force_quotes',
        )

    ### INITIALIZER ###

    def __init__(self, command=None, *arguments):
        if command is None:
            # TODO: Generalize these arbitrary default arguments away.
            command = 'draw-circle'
            assert len(arguments) == 0
        assert isinstance(command, str) \
            and len(command) and command.find(' ') == -1
        self._command = command
        self._args = tuple(arguments)
        self._force_quotes = False

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a markup command with command and
        arguments equal to those of this markup command. Otherwise false.

        ..  container:: example

            ::

                >>> command_1 = markuptools.MarkupCommand('box')
                >>> command_2 = markuptools.MarkupCommand('box')
                >>> command_3 = markuptools.MarkupCommand('line')

            ::
            
                >>> command_1 == command_1
                True
                >>> command_1 == command_2
                True
                >>> command_1 == command_3
                False
                >>> command_2 == command_1
                True
                >>> command_2 == command_2
                True
                >>> command_2 == command_3
                False
                >>> command_3 == command_1
                False
                >>> command_3 == command_2
                False
                >>> command_3 == command_3
                True

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            if self.command == argument.command:
                if self.arguments == argument.arguments:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats markup command.

        ..  container:: example

            Prints storage format:

            ::

                >>> circle = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
                >>> square = markuptools.MarkupCommand('rounded-box', 'hello?')
                >>> line = markuptools.MarkupCommand('line', [square, 'wow!'])
                >>> rotate = markuptools.MarkupCommand('rotate', 60, line)
                >>> combine = markuptools.MarkupCommand('combine', rotate, circle)

            ::

                >>> print(format(combine, 'storage'))
                markuptools.MarkupCommand(
                    'combine',
                    markuptools.MarkupCommand(
                        'rotate',
                        60,
                        markuptools.MarkupCommand(
                            'line',
                            [
                                markuptools.MarkupCommand(
                                    'rounded-box',
                                    'hello?'
                                    ),
                                'wow!',
                                ]
                            )
                        ),
                    markuptools.MarkupCommand(
                        'draw-circle',
                        2.5,
                        0.1,
                        False
                        )
                    )

        ..  container:: example

            Prints LilyPond format:

            ::

                >>> circle = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
                >>> square = markuptools.MarkupCommand('rounded-box', 'hello?')
                >>> line = markuptools.MarkupCommand('line', [square, 'wow!'])
                >>> rotate = markuptools.MarkupCommand('rotate', 60, line)
                >>> combine = markuptools.MarkupCommand('combine', rotate, circle)

            ::

                >>> print(format(combine, 'lilypond'))
                \combine
                    \rotate
                        #60
                        \line
                            {
                                \rounded-box
                                    hello?
                                wow!
                            }
                    \draw-circle
                        #2.5
                        #0.1
                        ##f

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        elif format_specification == 'lilypond':
            return self._get_lilypond_format()
        return str(self)

    def __hash__(self):
        r'''Hashes markup command.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(MarkupCommand, self).__hash__()

    def __repr__(self):
        r'''Gets markup command interpreter representation.

        ..  container:: example

            Interpreter representation is evaluable.

            ::

                >>> command = markuptools.MarkupCommand('hspace', 0)
                >>> command
                markuptools.MarkupCommand(
                    'hspace',
                    0
                    )

            ::

                >>> eval(repr(command))
                markuptools.MarkupCommand(
                    'hspace',
                    0
                    )

        Returns string.
        '''
        superclass = super(MarkupCommand, self)
        return superclass.__format__()

    def __str__(self):
        r'''Gets string representation of markup command.

        ..  container:: example

            ::

                >>> circle = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
                >>> square = markuptools.MarkupCommand('rounded-box', 'hello?')
                >>> line = markuptools.MarkupCommand('line', [square, 'wow!'])
                >>> rotate = markuptools.MarkupCommand('rotate', 60, line)
                >>> combine = markuptools.MarkupCommand('combine', rotate, circle)

            ::

                >>> print(str(combine))
                \combine
                    \rotate
                        #60
                        \line
                            {
                                \rounded-box
                                    hello?
                                wow!
                            }
                    \draw-circle
                        #2.5
                        #0.1
                        ##f

        Returns string.
        '''
        return self._get_lilypond_format()

    ### PRIVATE METHODS ###

    def _escape_string(self, string):
        if -1 == string.find(' '):
            return string
        string = repr(string)
        if string.startswith("'") and string.endswith("'"):
            string = string.replace('"', '\"')
            string = '"' + string[1:]
            string = string[:-1] + '"'
        return string

    def _get_format_pieces(self):
        def recurse(iterable):
            result = []
            for x in iterable:
                if isinstance(x, (list, tuple)):
                    result.append('{')
                    result.extend(recurse(x))
                    result.append('}')
                elif isinstance(x, schemetools.Scheme):
                    result.append(format(x))
                elif hasattr(x, '_get_format_pieces'):
                    result.extend(x._get_format_pieces())
                elif isinstance(x, str) and '\n' in x:
                    result.append('#"')
                    result.extend(x.splitlines())
                    result.append('"')
                else:
                    formatted = schemetools.Scheme.format_scheme_value(
                        x,
                        force_quotes=self.force_quotes,
                        )
                    if isinstance(x, str):
                        result.append(formatted)
                    else:
                        result.append('#{}'.format(formatted))
            return ['{}{}'.format(indent, x) for x in result]
        from abjad.tools import systemtools
        indent = systemtools.LilyPondFormatManager.indent
        parts = [r'\{}'.format(self.command)]
        parts.extend(recurse(self.arguments))
        return parts

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_args_values=(self.command,) + self.arguments,
            storage_format_kwargs_names=[],
            )

    def _get_lilypond_format(self):
        return '\n'.join(self._get_format_pieces())

    ### PUBLIC PROPERTIES ###

    @property
    def arguments(self):
        r'''Gets markup command arguments.

        ..  container:: example

            ::

                >>> arguments = ('draw-circle', 1, 0.1, False)
                >>> command = markuptools.MarkupCommand(*arguments)
                >>> command.arguments
                (1, 0.1, False)

        Returns tuple.
        '''
        return self._args

    # TODO: change to MarkupCommand.name
    @property
    def command(self):
        r'''Gets markup command name.

        ..  container:: example

            ::

                >>> arguments = ('draw-circle', 1, 0.1, False)
                >>> command = markuptools.MarkupCommand(*arguments)
                >>> command.command
                'draw-circle'

        Returns string.
        '''
        return self._command

    @property
    def force_quotes(self):
        r'''Is true when markup command should force quotes around arguments.
        Otherwise false.

        ..  container:: example

            Here's a markup command formatted in the usual way without forced
            quotes:

            ::

                >>> lines = ['foo', 'bar blah', 'baz']
                >>> command = markuptools.MarkupCommand('column', lines)
                >>> markup = Markup(command)

            ::

                >>> f(markup)
                \markup {
                    \column
                        {
                            foo
                            "bar blah"
                            baz
                        }
                    }

            The markup command forces quotes around only the spaced string
            ``'bar blah'``.

        ..  container:: example

            Here's the same markup command with forced quotes:

                >>> lines = ['foo', 'bar blah', 'baz']
                >>> command = markuptools.MarkupCommand('column', lines)
                >>> command.force_quotes = True
                >>> markup = Markup(command)

            ::

                >>> f(markup)
                \markup {
                    \column
                        {
                            "foo"
                            "bar blah"
                            "baz"
                        }
                    }

            The markup command forces quotes around all strings.

        The rendered result of forced and unforced quotes is the same.

        Defaults to false.

        Returns true or false.
        '''
        return self._force_quotes

    @force_quotes.setter
    def force_quotes(self, argument):
        assert isinstance(argument, bool), repr(argument)
        self._force_quotes = argument

    ### PUBLIC METHODS ###

    @staticmethod
    def combine_markup_commands(*commands):
        r'''Combines markup command and / or strings.

        LilyPond's '\combine' markup command can only take two arguments, so in
        order to combine more than two stencils, a cascade of '\combine'
        commands must be employed.  `combine_markup_commands` simplifies this
        process.

        ..  container:: example

            ::

                >>> markup_a = markuptools.MarkupCommand(
                ...     'draw-circle',
                ...     4,
                ...     0.4,
                ...     False,
                ...     )
                >>> markup_b = markuptools.MarkupCommand(
                ...     'filled-box',
                ...     schemetools.SchemePair(-4, 4),
                ...     schemetools.SchemePair(-0.5, 0.5), 1)
                >>> markup_c = "some text"

            ::

                >>> markup = markuptools.MarkupCommand.combine_markup_commands(
                ...     markup_a,
                ...     markup_b,
                ...     markup_c,
                ...     )
                >>> result = format(markup, 'lilypond')

            ::

                >>> print(result)
                \combine \combine \draw-circle #4 #0.4 ##f
                    \filled-box #'(-4 . 4) #'(-0.5 . 0.5) #1 "some text"

        Returns a markup command instance, or a string if that was the only
        argument.
        '''
        from abjad.tools import markuptools

        assert len(commands)
        assert all(
            isinstance(command, (markuptools.MarkupCommand, str))
            for command in commands
            )

        if 1 == len(commands):
            return commands[0]

        combined = MarkupCommand('combine', commands[0], commands[1])
        for command in commands[2:]:
            combined = MarkupCommand('combine', combined, command)
        return combined



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_NoteHead_written_pitch_01():
    r'''Set Note head pitch with integer.
    '''

    note = Note(13, (1, 4))
    note.note_head.written_pitch = 14

    "NoteHead(d'')"

    assert format(note.note_head) == "d''"
    assert note.note_head.written_pitch.numbered_pitch._pitch_number == 14


def test_scoretools_NoteHead_written_pitch_02():
    r'''Set Note head pitch with pitch.
    '''

    note = Note(13, (1, 4))
    note.note_head.written_pitch = NamedPitch(14)

    "NoteHead(d'')"

    assert format(note.note_head) == "d''"
    assert note.note_head.written_pitch.numbered_pitch._pitch_number == 14


def test_scoretools_NoteHead_written_pitch_03():
    r'''Can not set note-head pitch to none.
    '''

    note = Note(13, (1, 4))

    assert pytest.raises(Exception, 'note.note_head.written_pitch = None')


def test_scoretools_NoteHead_written_pitch_04():
    r'''Set note-head pitch from another note or note-head.
    Make sure this does not cause reference problems.
    '''

    n1 = Note(12, (1, 4))
    n2 = Note(14, (1, 4))
    n1.written_pitch = n2.written_pitch

    assert n1.written_pitch == NamedPitch(14)
    assert n2.written_pitch == NamedPitch(14)
    assert n1.written_pitch is not n2.written_pitch



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


# TODO: implement __illustrate__
class NumberedPitchClassColorMap(AbjadValueObject):
    '''Numbered pitch-class color map.

    ..  container:: example

        Maps pitch-classes to red, green and blue:

        ::

            >>> pitches = [
            ...     [-8, 2, 10, 21],
            ...     [0, 11, 32, 41],
            ...     [15, 25, 42, 43],
            ...     ]
            >>> colors = ['red', 'green', 'blue']
            >>> color_map = pitchtools.NumberedPitchClassColorMap(pitches, colors)

    Numbered pitch-class color maps are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_color_dictionary',
        '_colors',
        '_pitch_iterables',
        )

    ### INITIALIZER ###

    def __init__(self, pitch_iterables=None, colors=None):
        pitch_iterables = pitch_iterables or []
        colors = colors or []
        assert len(pitch_iterables) == len(colors)
        self._pitch_iterables = pitch_iterables
        self._colors = colors
        self._color_dictionary = {}
        self._initialize_color_dictionary()

    ### SPECIAL METHODS ###

    def __getitem__(self, pitch_class):
        r'''Gets `pitch_class` color.

        ::

            >>> color_map[11]
            'green'

        Returns string.
        '''
        from abjad.tools import pitchtools
        pitch_class = pitchtools.NumberedPitchClass(pitch_class)
        return self._color_dictionary[pitch_class.pitch_class_number]

    ### PRIVATE METHODS ###

    def _initialize_color_dictionary(self):
        from abjad.tools import pitchtools
        for pitch_iterable, color in zip(self.pitch_iterables, self.colors):
            for pitch in pitch_iterable:
                pc = pitchtools.NumberedPitchClass(pitch)
                if pc.pitch_class_number in list(self._color_dictionary.keys()):
                    print(pc, list(self._color_dictionary.keys()))
                    message = 'duplicated pitch-class in color map: {!r}.'
                    message = message.format(pc)
                    raise KeyError(message)
                self._color_dictionary[pc.pitch_class_number] = color

    ### PUBLIC PROPERTIES ###

    @property
    def colors(self):
        r'''Colors of color map.

        ::

            >>> color_map.colors
            ['red', 'green', 'blue']

        Returns list.
        '''
        return self._colors

    @property
    def is_twelve_tone_complete(self):
        r'''Is true when color map contains all 12-ET pitch-classes.

        ::

            >>> color_map.is_twelve_tone_complete
            True

        Return boolean.
        '''
        pcs = range(12)
        return set(pcs).issubset(set(self._color_dictionary.keys()))

    @property
    def is_twenty_four_tone_complete(self):
        r'''Is true when color map contains all 24-ET pitch-classes.

        ::

            >>> color_map.is_twenty_four_tone_complete
            False

        Return boolean.
        '''
        pcs = [x / 2.0 for x in range(24)]
        pcs = [int(x) if int(x) == x else x for x in pcs]
        return set(pcs).issubset(set(self._color_dictionary.keys()))

    @property
    def pairs(self):
        r'''Pairs of color map.

        ::

            >>> for pair in color_map.pairs:
            ...     pair
            (0, 'green')
            (1, 'blue')
            (2, 'red')
            (3, 'blue')
            (4, 'red')
            (5, 'green')
            (6, 'blue')
            (7, 'blue')
            (8, 'green')
            (9, 'red')
            (10, 'red')
            (11, 'green')

        Returns list.
        '''
        items = list(self._color_dictionary.items())
        return list(sorted(items))

    @property
    def pitch_iterables(self):
        r'''Pitch interables of color map.

        ::

            >>> color_map.pitch_iterables
            [[-8, 2, 10, 21], [0, 11, 32, 41], [15, 25, 42, 43]]

        Returns ?
        '''
        return self._pitch_iterables

    ### PUBLIC METHODS ###

    def get(self, key, alternative=None):
        r'''Gets `key` from color map.

        ::

            >>> color_map.get(11)
            'green'

        Returns `alternative` when `key` is not found.

        Returns string.
        '''
        try:
            return self[key]
        except (KeyError, TypeError, ValueError):
            return alternative



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Leaf_written_duration_01():
    r'''Leaf durations can go up to 'maxima...': duration < (16, 1).
    '''

    note = Note(1, 2)

    assert format(note) == "cs'\\breve"
    note.written_duration = Duration(3)
    assert format(note) == "cs'\\breve."
    note.written_duration = Duration(4)
    assert format(note) == "cs'\\longa"
    note.written_duration = Duration(6)
    assert format(note) == "cs'\\longa."
    note.written_duration = Duration(7)
    assert format(note) == "cs'\\longa.."
    note.written_duration = Duration(8)
    assert format(note) == "cs'\\maxima"
    note.written_duration = Duration(12)
    assert format(note) == "cs'\\maxima."
    note.written_duration = Duration(14)
    assert format(note) == "cs'\\maxima.."
    note.written_duration = Duration(15)
    assert format(note) == "cs'\\maxima..."
    assert pytest.raises(AssignabilityError, 'Note(1, 16)')



# -*- coding: utf-8 -*-
import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class TimeRelation(AbjadObject):
    r'''A time relation.

    Time relations are immutable.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Time relations'

    __slots__ = (
        '_inequality',
        )

    ### INITIALIZER ###

    def __init__(self, inequality=None):
        from abjad.tools import timespantools
        if not inequality:
            inequality = timespantools.CompoundInequality([
                'timespan_1.start_offset < timespan_2.start_offset',
                ])
        assert isinstance(
            inequality, timespantools.CompoundInequality), repr(inequality)
        self._inequality = inequality

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self):
        r'''Evaluates time relation.

        Returns true or false.
        '''
        pass

    @abc.abstractmethod
    def __eq__(self, argument):
        r'''Is true when `argument` is a equal-valued time relation.
        Otherwise false.

        Returns true or false.
        '''
        pass

    def __format__(self, format_specification=''):
        r'''Formats time relation.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __hash__(self):
        r'''Hashes time relation.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(TimeRelation, self).__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def inequality(self):
        r'''Time relation inequality.

        Return ineqality.
        '''
        return self._inequality

    @abc.abstractproperty
    def is_fully_loaded(self):
        r'''Is true when both time relation terms are not none.
        Otherwise false:

        Returns true or false.
        '''
        pass

    @abc.abstractproperty
    def is_fully_unloaded(self):
        r'''Is true when both time relation terms are none.
        Otherwise false:

        Returns true or false.
        '''
        pass



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def fill_measures_with_full_measure_spacer_skips(argument, iterctrl=None):
    r'''Fill measures in `argument` with full-measure spacer skips.
    '''
    from abjad.tools import scoretools

    if iterctrl is None:
        iterctrl = lambda measure, i: True

    measures = iterate(argument).by_class(scoretools.Measure)
    for i, measure in enumerate(measures):
        if iterctrl(measure, i):
            skip = scoretools.Skip(1)
            # allow zero-update iteration
            time_signature = measure.time_signature
            duration = time_signature.duration
            if measure.implicit_scaling:
                implied_prolation = time_signature.implied_prolation
                multiplier = duration.__div__(implied_prolation)
            else:
                multiplier = durationtools.Multiplier(duration)
            attach(multiplier, skip)
            measure[:] = [skip]
            for spanner in measure._get_spanners():
                spanner._remove(measure)



# -*- coding: utf-8 -*-
import inspect
import pickle
import pytest
from abjad.tools import abjadbooktools
from abjad.tools import documentationtools
from abjad.tools import lilypondparsertools
from abjad.tools import quantizationtools
from abjad.tools import rhythmtreetools
from abjad.tools import selectiontools
from abjad.tools import systemtools
from abjad.tools import tonalanalysistools


ignored_classes = (
    abjadbooktools.CodeBlock,
    abjadbooktools.CodeOutputProxy,
    abjadbooktools.GraphvizOutputProxy,
    abjadbooktools.LilyPondOutputProxy,
    documentationtools.InheritanceGraph,
    lilypondparsertools.LilyPondParser,
    lilypondparsertools.SchemeParser,
    lilypondparsertools.ReducedLyParser,
    quantizationtools.ParallelJobHandlerWorker,
    rhythmtreetools.RhythmTreeParser,
    selectiontools.Selection,
    selectiontools.Descendants,
    selectiontools.LogicalTie,
    selectiontools.Lineage,
    selectiontools.Parentage,
    selectiontools.Selection,
    selectiontools.Selection,
    selectiontools.VerticalMoment,
    systemtools.LilyPondFormatBundle,
    systemtools.LilyPondFormatBundle.SlotContributions,
    systemtools.RedirectedStreams,
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    tonalanalysistools.TonalAnalysisAgent,
    tonalanalysistools.RootedChordClass
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.parametrize('class_', classes)
def test_abjad_pickle_01(class_):
    r'''All storage-formattable classes are pickable.
    '''
    if (
        '_get_storage_format_specification' not in dir(class_) or
        '_get_format_specification' not in dir(class_)
        ):
        return
    if inspect.isabstract(class_):
        return
    instance_one = class_()
    pickle_string = pickle.dumps(instance_one)
    instance_two = pickle.loads(pickle_string)
    instance_one_format = format(instance_one, 'storage')
    instance_two_format = format(instance_two, 'storage')
    assert instance_one_format == instance_two_format



# -*- coding: utf-8 -*-
import pytest
from abjad import *
import pytest


def test_pitchtools_NumberedPitchClassColorMap_01():
    r'''Test basic pc color map attributes.
    '''

    pitches = [[-8, 2, 10, 21], [0, 11, 32, 41], [15, 25, 42, 43]]
    colors = ['red', 'green', 'blue']
    pcm = pitchtools.NumberedPitchClassColorMap(pitches, colors)

    "PitchClassColorMap([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11])"

    assert pcm.colors == ['red', 'green', 'blue']
    assert pcm.pairs == [
        (0, 'green'),
        (1, 'blue'),
        (2, 'red'),
        (3, 'blue'),
        (4, 'red'),
        (5, 'green'),
        (6, 'blue'),
        (7, 'blue'),
        (8, 'green'),
        (9, 'red'),
        (10, 'red'),
        (11, 'green'),
        ]
    assert pcm.pitch_iterables == [
        [-8, 2, 10, 21],
        [0, 11, 32, 41],
        [15, 25, 42, 43],
        ]
    assert pcm.is_twelve_tone_complete
    assert not pcm.is_twenty_four_tone_complete


def test_pitchtools_NumberedPitchClassColorMap_02():
    r'''Test pc color map get item variations.
    '''

    pitches = [[-8, 2, 10, 21], [0, 11, 32, 41], [15, 25, 42, 43]]
    colors = ['red', 'green', 'blue']
    pcm = pitchtools.NumberedPitchClassColorMap(pitches, colors)

    "PitchClassColorMap([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11])"

    assert pcm[1] == 'blue'
    assert pcm[pitchtools.NumberedPitchClass(1)] == 'blue'
    assert pcm[pitchtools.NamedPitch(1)] == 'blue'

    assert pcm[13] == 'blue'
    assert pcm[pitchtools.NumberedPitchClass(13)] == 'blue'
    assert pcm[pitchtools.NamedPitch(13)] == 'blue'

    assert pytest.raises(Exception, "pcm['foo']")

    assert pcm.get(1) == 'blue'
    assert pcm.get('foo') is None
    assert pcm.get('foo', 'blah') == 'blah'



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchRange___le___01():

    pitch_range = pitchtools.PitchRange.from_pitches(-39, 48)

    assert not pitch_range <= -99
    assert not pitch_range <= -39
    assert not pitch_range <= 0
    assert pitch_range <= 48
    assert pitch_range <= 99


def test_pitchtools_PitchRange___le___02():

    pitch_range = pitchtools.PitchRange.from_pitches(-39, 48)

    assert not pitch_range <= NamedPitch(-99)
    assert not pitch_range <= NamedPitch(-39)
    assert not pitch_range <= NamedPitch(0)
    assert pitch_range <= NamedPitch(48)
    assert pitch_range <= NamedPitch(99)



# -*- coding: utf-8 -*-
from abjad.tools.quantizationtools.QEvent import QEvent


class SilentQEvent(QEvent):
    r'''A ``QEvent`` which indicates the onset of a period of silence
    in a ``QEventSequence``.

        >>> q_event = quantizationtools.SilentQEvent(1000)
        >>> q_event
        SilentQEvent(offset=Offset(1000, 1))

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_attachments',
        )

    ### INITIALIZER ###

    def __init__(self, offset=0, attachments=None, index=None):
        QEvent.__init__(self, offset=offset, index=index)
        if attachments is None:
            attachments = ()
        else:
            attachments = tuple(attachments)
        self._attachments = attachments

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a silent q-event with offset, attachments and
        index equal to those of this silent q-event. Otherwise false.

        Returns true or false.
        '''
        if type(self) == type(argument) and \
            self._offset == argument._offset and \
            self._attachments == argument._attachments and \
            self._index == argument._index:
            return True
        return False

    def __hash__(self):
        r'''Hashes silent q-event.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(SilentQEvent, self).__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def attachments(self):
        r'''Attachments of silen q-event.
        '''
        return self._attachments



# -*- coding: utf-8 -*-


def select_first(n=1, inverted=None):
    r'''Makes pattern that matches the first `n` indices.

    ..  container:: example

        Selects first division for tie creation:

        ::

            >>> pattern = patterntools.select_first()

        ::

            >>> print(format(pattern))
            patterntools.Pattern(
                indices=[0],
                )

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker(
            ...     tie_specifier=rhythmmakertools.TieSpecifier(
            ...         tie_across_divisions=pattern,
            ...         use_messiaen_style_ties=True,
            ...         ),
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4. \repeatTie
                }
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
            }
            
    ..  container:: example

        Selects first two divisions for tie creation:

        ::

            >>> pattern = patterntools.select_first(n=2)

        ::

            >>> print(format(pattern))
            patterntools.Pattern(
                indices=[0, 1],
                )

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker(
            ...     tie_specifier=rhythmmakertools.TieSpecifier(
            ...         tie_across_divisions=pattern,
            ...         use_messiaen_style_ties=True,
            ...         ),
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4. \repeatTie
                }
                {
                    \time 7/16
                    c'4.. \repeatTie
                }
                {
                    \time 3/8
                    c'4.
                }
            }

    ..  container:: example

        Selects no divisions for tie creation:

        ::

            >>> pattern = patterntools.select_first(n=0)

        ::

            >>> print(format(pattern))
            patterntools.Pattern(
                indices=[],
                )

        ::

            >>> maker = rhythmmakertools.NoteRhythmMaker(
            ...     tie_specifier=rhythmmakertools.TieSpecifier(
            ...         tie_across_divisions=pattern,
            ...         use_messiaen_style_ties=True,
            ...         ),
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
            }

    Returns pattern.
    '''
    from abjad.tools import patterntools
    indices = list(range(n))
    return patterntools.Pattern(
        indices=indices,
        inverted=inverted,
        )



# -*- coding: utf-8 -*-
from abjad.tools.scoretools.Container import Container


class GraceContainer(Container):
    r'''A grace container.

    .. container:: example

        **Example 1.** Grace notes:

        ::

            >>> voice = Voice("c'4 d'4 e'4 f'4")
            >>> grace_notes = [Note("c'16"), Note("d'16")]
            >>> grace_container = scoretools.GraceContainer(
            ...     grace_notes,
            ...     kind='grace',
            ...     )
            >>> attach(grace_container, voice[1])
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice {
                c'4
                \grace {
                    c'16
                    d'16
                }
                d'4
                e'4
                f'4
            }

    ..  container:: example

        **Example 2.** After-grace notes:

        ::

            >>> voice = Voice("c'4 d'4 e'4 f'4")
            >>> grace_notes = [Note("c'16"), Note("d'16")]
            >>> grace_container = scoretools.GraceContainer(
            ...     grace_notes,
            ...     kind='after',
            ...     )
            >>> attach(grace_container, voice[1])
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice {
                c'4
                \afterGrace
                d'4
                {
                    c'16
                    d'16
                }
                e'4
                f'4
            }

    Fill grace containers with notes, rests or chords.

    Attach grace containers to notes, rests or chords.

    ..  container:: example

        **Example 3.** Detaches all grace containers:

        ::

            >>> voice = Voice("c'4 d'4 e'4 f'4")
            >>> note = Note("cs'16")
            >>> grace_container = scoretools.GraceContainer(
            ...     [note],
            ...     kind='grace',
            ...     )
            >>> attach(grace_container, voice[1])
            >>> note = Note("ds'16")
            >>> after_grace_container = scoretools.GraceContainer(
            ...     [note],
            ...     kind='after',
            ...     )
            >>> attach(after_grace_container, voice[1])
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice {
                c'4
                \grace {
                    cs'16
                }
                \afterGrace
                d'4
                {
                    ds'16
                }
                e'4
                f'4
            }

        ::

            >>> detach(scoretools.GraceContainer, voice[1])
            (GraceContainer(), GraceContainer())
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice {
                c'4
                d'4
                e'4
                f'4
            }

    ..  container:: example

        **Example 4.** Detaches (proper) grace container but leaves after grace
        container attached:

        ::

            >>> voice = Voice("c'4 d'4 e'4 f'4")
            >>> note = Note("cs'16")
            >>> grace_container = scoretools.GraceContainer(
            ...     [note],
            ...     kind='grace',
            ...     )
            >>> attach(grace_container, voice[1])
            >>> note = Note("ds'16")
            >>> after_grace_container = scoretools.GraceContainer(
            ...     [note],
            ...     kind='after',
            ...     )
            >>> attach(after_grace_container, voice[1])
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice {
                c'4
                \grace {
                    cs'16
                }
                \afterGrace
                d'4
                {
                    ds'16
                }
                e'4
                f'4
            }

        ::

            >>> detach(grace_container, voice[1])
            (GraceContainer(),)
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice {
                c'4
                \afterGrace
                d'4
                {
                    ds'16
                }
                e'4
                f'4
            }

    ..  container:: example

        **Example 5.** Detaches after grace container but leaves (proper) grace
        container attached:

        ::

            >>> voice = Voice("c'4 d'4 e'4 f'4")
            >>> note = Note("cs'16")
            >>> grace_container = scoretools.GraceContainer(
            ...     [note],
            ...     kind='grace',
            ...     )
            >>> attach(grace_container, voice[1])
            >>> note = Note("ds'16")
            >>> after_grace_container = scoretools.GraceContainer(
            ...     [note],
            ...     kind='after',
            ...     )
            >>> attach(after_grace_container, voice[1])
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice {
                c'4
                \grace {
                    cs'16
                }
                \afterGrace
                d'4
                {
                    ds'16
                }
                e'4
                f'4
            }

        ::

            >>> detach(after_grace_container, voice[1])
            (GraceContainer(),)
            >>> show(voice) # doctest: +SKIP

        ..  doctest::

            >>> f(voice)
            \new Voice {
                c'4
                \grace {
                    cs'16
                }
                d'4
                e'4
                f'4
            }

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Containers'

    __slots__ = (
        '_carrier',
        '_kind',
        )

    _allowable_kinds = (
        'after',
        'grace',
        'acciaccatura',
        'appoggiatura',
        )

    ### INITIALIZER ###

    def __init__(self, music=None, kind='grace'):
        # self._carrier is a reference to the leaf carrying grace music
        self._carrier = None
        Container.__init__(self, music)
        self.kind = kind

    ### PRIVATE METHODS ###

    def _attach(self, leaf):
        from abjad.tools import scoretools
        if not isinstance(leaf, scoretools.Leaf):
            message = 'must attach to leaf: {!r}.'
            message = message.format(leaf)
            raise TypeError(message)
        if self.kind == 'after':
            leaf._after_grace = self
        else:
            leaf._grace = self
        self._carrier = leaf

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = Container._copy_with_indicators_but_without_children_or_spanners(
            self)
        new.kind = self.kind
        return new

    def _detach(self):
        if self._carrier is not None:
            carrier = self._carrier
            if self.kind == 'after':
                carrier._after_grace = None
            else:
                carrier._grace = None
            self._carrier = None
            self[:] = []
        return self

    def _format_open_brackets_slot(self, bundle):
        result = []
        kind = self.kind
        if kind == 'after':
            result.append([('grace_brackets', 'open'), ['{']])
        else:
            contributor = ('grace_brackets', 'open')
            contributions = [r'\{} {{'.format(kind)]
            result.append([contributor, contributions])
        return tuple(result)

    def _get_lilypond_format(self):
        self._update_now(indicators=True)
        return self._format_component()

    ### PUBLIC PROPERTIES ###

    @property
    def kind(self):
        r'''Gets and sets `kind`.

        .. container:: example

            **Example 1.** Grace notes:

            ::

                >>> voice = Voice("c'4 d'4 e'4 f'4")
                >>> grace_container = scoretools.GraceContainer(
                ...     [Note("e'16")],
                ...     kind='grace',
                ...     )
                >>> attach(grace_container, voice[1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    c'4
                    \grace {
                        e'16
                    }
                    d'4
                    e'4
                    f'4
                }

            LilyPond positions grace notes immediately before main notes.

            LilyPond formats grace notes with neither a slashed nor a slur.

        .. container:: example

            **Example 2.** Acciaccatura:

            ::

                >>> voice = Voice("c'4 d'4 e'4 f'4")
                >>> grace_container = scoretools.GraceContainer(
                ...     [Note("e'16")],
                ...     kind='acciaccatura',
                ...     )
                >>> attach(grace_container, voice[1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    c'4
                    \acciaccatura {
                        e'16
                    }
                    d'4
                    e'4
                    f'4
                }

            Acciaccaturas are played before the beat.

            LilyPond positions acciaccaturas immediately before main notes.

            LilyPond formats one-note acciaccaturas with a slashed stem and a
            slur.

            ..  note:: LilyPond fails to format multinote acciaccaturas
                with a slashed stem. This means that multinote
                acciaccaturas look exactly like appoggiaturas.

        .. container:: example

            **Example 3.** Appoggiatura:

            ::

                >>> voice = Voice("c'4 d'4 e'4 f'4")
                >>> grace_container = scoretools.GraceContainer(
                ...     [Note("e'16")],
                ...     kind='appoggiatura',
                ...     )
                >>> attach(grace_container, voice[1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    c'4
                    \appoggiatura {
                        e'16
                    }
                    d'4
                    e'4
                    f'4
                }

            Appoggiaturas are played on the beat.

            LilyPond positions appoggiaturas immediately before main notes.

            LilyPond formats appoggiaturas with a slur but without a slashed
            stem.

        .. container:: example

            **Example 4.** After-grace notes:

            ::

                >>> voice = Voice("c'4 d'4 e'4 f'4")
                >>> string = '#(define afterGraceFraction (cons 15 16))'
                >>> command = indicatortools.LilyPondCommand(string)
                >>> attach(command, voice[0])
                >>> grace_container = scoretools.GraceContainer(
                ...     [Note("g'16")],
                ...     kind='after',
                ...     )
                >>> attach(grace_container, voice[-1])
                >>> show(voice) # doctest: +SKIP

            ..  doctest::

                >>> f(voice)
                \new Voice {
                    #(define afterGraceFraction (cons 15 16))
                    c'4
                    d'4
                    e'4
                    \afterGrace
                    f'4
                    {
                        g'16
                    }
                }

            After-grace notes are played at the very end of the note they
            follow.

            Use after-grace notes when you need to end a piece of music with
            grace notes.

            LilyPond positions after-grace notes at a point 3/4 of the way
            after the note they follow. The resulting spacing is usually too
            loose.

            Customize aftterGraceFraction as shown above.

        Defaults to ``'grace'``.

        Set to ``'grace'``, ``'acciaccatura'``, ``'appoggiatura'`` or
        ``'after'``.

        Returns ``'grace'``, ``'acciaccatura'``, ``'appoggiatura'`` or
        ``'after'``.
        '''
        return self._kind

    @kind.setter
    def kind(self, arg):
        if arg not in self._allowable_kinds:
            message = 'unknown grace container kind: {!r}.'
            message = message.format(arg)
            raise Exception(message)
        self._kind = arg



# -*- coding: utf-8 -*-
from abjad.tools import durationtools


def make_repeated_notes(count, duration=durationtools.Duration(1, 8)):
    r'''Make `count` repeated notes with note-head-assignable `duration`:

    ::

        >>> scoretools.make_repeated_notes(4)
        Selection([Note("c'8"), Note("c'8"), Note("c'8"), Note("c'8")])

    Make `count` repeated logical ties with tied `duration`:

    ::

        >>> notes = scoretools.make_repeated_notes(2, (5, 16))
        >>> voice = Voice(notes)

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            c'4 ~
            c'16
            c'4 ~
            c'16
        }

    Make ad hoc tuplet holding `count` repeated notes with
    non-power-of-two `duration`:

    ::

        >>> scoretools.make_repeated_notes(3, (1, 12))
        Selection([Tuplet(Multiplier(2, 3), "c'8 c'8 c'8")])

    Set pitch of all notes created to middle C.

    Returns list of zero or more newly constructed notes or list
    of one newly constructed tuplet.
    '''
    from abjad.tools import scoretools

    return scoretools.make_notes([0] * count, [duration])



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import systemtools
from abjad.tools.quantizationtools.QSchema import QSchema


class BeatwiseQSchema(QSchema):
    r'''Concrete ``QSchema`` subclass which treats "beats" as its time-step
    unit.

    ::

        >>> q_schema = quantizationtools.BeatwiseQSchema()

    Without arguments, it uses smart defaults:

    ::

        >>> print(format(q_schema, 'storage'))
        quantizationtools.BeatwiseQSchema(
            beatspan=durationtools.Duration(1, 4),
            search_tree=quantizationtools.UnweightedSearchTree(
                definition={
                    2: {
                        2: {
                            2: {
                                2: None,
                                },
                            3: None,
                            },
                        3: None,
                        5: None,
                        7: None,
                        },
                    3: {
                        2: {
                            2: None,
                            },
                        3: None,
                        5: None,
                        },
                    5: {
                        2: None,
                        3: None,
                        },
                    7: {
                        2: None,
                        },
                    11: None,
                    13: None,
                    },
                ),
            tempo=indicatortools.Tempo(
                reference_duration=durationtools.Duration(1, 4),
                units_per_minute=60,
                ),
            )

    Each time-step in a ``BeatwiseQSchema`` is composed of three settings:

        * ``beatspan``
        * ``search_tree``
        * ``tempo``

    These settings can be applied as global defaults for the schema via keyword
    arguments, which persist until overridden:

    ::

        >>> beatspan = Duration(5, 16)
        >>> search_tree = quantizationtools.UnweightedSearchTree({7: None})
        >>> tempo = Tempo((1, 4), 54)
        >>> q_schema = quantizationtools.BeatwiseQSchema(
        ...     beatspan=beatspan,
        ...     search_tree=search_tree,
        ...     tempo=tempo,
        ...     )

    The computed value at any non-negative time-step can be found by
    subscripting:

    ::

        >>> index = 0
        >>> for key, value in sorted(q_schema[index].items()):
        ...     print('{}:'.format(key), value)
        ...
        beatspan: 5/16
        search_tree: UnweightedSearchTree(definition={7: None})
        tempo: 4=54

    ::

        >>> index = 1000
        >>> for key, value in sorted(q_schema[index].items()):
        ...     print('{}:'.format(key), value)
        ...
        beatspan: 5/16
        search_tree: UnweightedSearchTree(definition={7: None})
        tempo: 4=54

    Per-time-step settings can be applied in a variety of ways.

    Instantiating the schema via ``*arguments`` with a series of either
    ``BeatwiseQSchemaItem`` instances, or dictionaries which could be used to
    instantiate ``BeatwiseQSchemaItem`` instances, will apply those settings
    sequentially, starting from time-step ``0``:

    ::

        >>> a = {'beatspan': Duration(5, 32)}
        >>> b = {'beatspan': Duration(3, 16)}
        >>> c = {'beatspan': Duration(1, 8)}

    ::

        >>> q_schema = quantizationtools.BeatwiseQSchema(a, b, c)

    ::

        >>> q_schema[0]['beatspan']
        Duration(5, 32)

    ::

        >>> q_schema[1]['beatspan']
        Duration(3, 16)

    ::

        >>> q_schema[2]['beatspan']
        Duration(1, 8)

    ::

        >>> q_schema[3]['beatspan']
        Duration(1, 8)

    Similarly, instantiating the schema from a single dictionary, consisting
    of integer:specification pairs, or a sequence via ``*arguments`` of (integer,
    specification) pairs, allows for applying settings to  non-sequential
    time-steps:

    ::

        >>> a = {'search_tree': quantizationtools.UnweightedSearchTree({2: None})}
        >>> b = {'search_tree': quantizationtools.UnweightedSearchTree({3: None})}

    ::

        >>> settings = {
        ...     2: a,
        ...     4: b,
        ... }

    ::

        >>> q_schema = quantizationtools.MeasurewiseQSchema(settings)

    ::

        >>> print(format(q_schema[0]['search_tree']))
        quantizationtools.UnweightedSearchTree(
            definition={
                2: {
                    2: {
                        2: {
                            2: None,
                            },
                        3: None,
                        },
                    3: None,
                    5: None,
                    7: None,
                    },
                3: {
                    2: {
                        2: None,
                        },
                    3: None,
                    5: None,
                    },
                5: {
                    2: None,
                    3: None,
                    },
                7: {
                    2: None,
                    },
                11: None,
                13: None,
                },
            )

    ::

        >>> print(format(q_schema[1]['search_tree']))
        quantizationtools.UnweightedSearchTree(
            definition={
                2: {
                    2: {
                        2: {
                            2: None,
                            },
                        3: None,
                        },
                    3: None,
                    5: None,
                    7: None,
                    },
                3: {
                    2: {
                        2: None,
                        },
                    3: None,
                    5: None,
                    },
                5: {
                    2: None,
                    3: None,
                    },
                7: {
                    2: None,
                    },
                11: None,
                13: None,
                },
            )

    ::

        >>> q_schema[2]['search_tree']
        UnweightedSearchTree(definition={2: None})

    ::

        >>> q_schema[3]['search_tree']
        UnweightedSearchTree(definition={2: None})

    ::

        >>> q_schema[4]['search_tree']
        UnweightedSearchTree(definition={3: None})

    ::

        >>> q_schema[1000]['search_tree']
        UnweightedSearchTree(definition={3: None})

    The following is equivalent to the above schema definition:

    ::

        >>> q_schema = quantizationtools.MeasurewiseQSchema(
        ...     (2, {'search_tree': quantizationtools.UnweightedSearchTree({2: None})}),
        ...     (4, {'search_tree': quantizationtools.UnweightedSearchTree({3: None})}),
        ...     )

    Return ``BeatwiseQSchema`` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_beatspan',
        '_items',
        '_lookups',
        '_search_tree',
        '_tempo',
        )

    _keyword_argument_names = (
        'beatspan',
        'search_tree',
        'tempo',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments, **keywords):
        from abjad.tools import quantizationtools
        self._beatspan = durationtools.Duration(
            keywords.get('beatspan',
                (1, 4)))
        search_tree = keywords.get('search_tree',
            quantizationtools.UnweightedSearchTree())
        assert isinstance(search_tree, quantizationtools.SearchTree)
        self._search_tree = search_tree
        tempo = keywords.get('tempo', ((1, 4), 60))
        if isinstance(tempo, tuple):
            tempo = indicatortools.Tempo(*tempo)
        self._tempo = tempo
        QSchema.__init__(self, *arguments, **keywords)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=self.items or (),
            storage_format_kwargs_names=[
                'beatspan',
                'search_tree',
                'tempo',
                ],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def beatspan(self):
        r'''Default beatspan of beatwise q-schema.
        '''
        return self._beatspan

    @property
    def item_class(self):
        r'''The schema's item class.

        Returns beatwise q-schema item.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.BeatwiseQSchemaItem

    @property
    def target_class(self):
        r'''Target class of beatwise q-schema.

        Returns beatwise q-target.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.BeatwiseQTarget

    @property
    def target_item_class(self):
        r'''Target item class of beatwise q-schema.

        Returns q-target beat.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.QTargetBeat



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Leaf__get_leaf_index_01():
    r'''Leaves in staff number correctly.
    '''

    staff = Staff("c'8 d'8 e'8")
    assert staff[0]._get_leaf_index() == 0
    assert staff[1]._get_leaf_index() == 1
    assert staff[2]._get_leaf_index() == 2


def test_scoretools_Leaf__get_leaf_index_02():
    r'''Leaves in measure in staff number correctly.
    '''

    staff = Staff([Measure((3, 8), "c'8 d'8 e'8")])
    leaves = select(staff).by_leaf()
    assert leaves[0]._get_leaf_index() == 0
    assert leaves[1]._get_leaf_index() == 1
    assert leaves[2]._get_leaf_index() == 2


def test_scoretools_Leaf__get_leaf_index_03():
    r'''Leaves in multiple measures in staff number corretly.
    '''

    staff = Staff(Measure((2, 8), "c'8 d'8") * 3)
    leaves = select(staff).by_leaf()
    assert leaves[0]._get_leaf_index() == 0
    assert leaves[1]._get_leaf_index() == 1
    assert leaves[2]._get_leaf_index() == 2
    assert leaves[3]._get_leaf_index() == 3
    assert leaves[4]._get_leaf_index() == 4
    assert leaves[5]._get_leaf_index() == 5


def test_scoretools_Leaf__get_leaf_index_04():
    r'''Orphan leaves number correctly.
    '''

    note = Note("c'4")
    assert note._get_leaf_index() == 0


def test_scoretools_Leaf__get_leaf_index_05():
    r'''Leaves number correctly after contents rotation.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")

    assert staff[0]._get_leaf_index() == 0
    assert staff[1]._get_leaf_index() == 1
    assert staff[2]._get_leaf_index() == 2
    assert staff[3]._get_leaf_index() == 3

    staff[:] = (staff[-2:] + staff[:2])

    r'''
    \new Staff {
        e'8
        f'8
        c'8
        d'8
    }
    '''

    assert inspect_(staff).is_well_formed()
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            e'8
            f'8
            c'8
            d'8
        }
        '''
        )

    assert staff[0]._get_leaf_index() == 0
    assert staff[1]._get_leaf_index() == 1
    assert staff[2]._get_leaf_index() == 2
    assert staff[3]._get_leaf_index() == 3



# -*- coding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools.quantizationtools.GraceHandler import GraceHandler


class ConcatenatingGraceHandler(GraceHandler):
    r'''Concrete ``GraceHandler`` subclass which concatenates all but the
    final ``QEvent`` attached to a ``QGrid`` offset into a ``GraceContainer``,
    using a fixed leaf duration ``duration``.

    When called, it returns pitch information of final ``QEvent``, and the
    generated ``GraceContainer``, if any.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_grace_duration',
        )

    ### INITIALIZER ###

    def __init__(self, grace_duration=None):
        if grace_duration is None:
            grace_duration = (1, 16)
        grace_duration = durationtools.Duration(grace_duration)
        assert grace_duration.has_power_of_two_denominator
        self._grace_duration = grace_duration

    ### SPECIAL METHODS ###

    def __call__(self, q_events):
        r'''Calls concatenating grace handler.
        '''
        from abjad.tools import quantizationtools

        grace_events, final_event = q_events[:-1], q_events[-1]

        if isinstance(final_event, quantizationtools.PitchedQEvent):
            pitches = final_event.pitches
        else:
            pitches = ()

        if grace_events:
            grace_container = scoretools.GraceContainer()
            for q_event in grace_events:
                if isinstance(q_event, quantizationtools.PitchedQEvent):
                    if len(q_event.pitches) == 1:
                        leaf = scoretools.Note(
                            q_event.pitches[0], self.grace_duration)
                    else:
                        leaf = scoretools.Chord(
                            q_event.pitches, self.grace_duration)
                else:
                    leaf = scoretools.Rest(self.grace_duration)
                grace_container.append(leaf)
        else:
            grace_container = None

        return pitches, grace_container

    ### PUBLIC PROPERTIES ###

    @property
    def grace_duration(self):
        r'''Grace duration of concantenating grace handler.

        Returns duration.
        '''
        return self._grace_duration



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Leaf__set_duration_01():
    r'''Change leaf to tied duration.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8
            f'8
        }
        '''
        )

    voice[1]._set_duration(Duration(5, 32))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ~
            d'32 ]
            e'8
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()

def test_scoretools_Leaf__set_duration_02():
    r'''Change tied leaf to tied value.
    Duplicate ties are not created.
    '''

    voice = Voice("c'8 c'8 c'8 c'8")
    tie = spannertools.Tie()
    attach(tie, voice[:2])
    beam = Beam()
    attach(beam, voice[:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 ~ [
            c'8 ]
            c'8
            c'8
        }
        '''
        )

    voice[1]._set_duration(Duration(5, 32))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 ~ [
            c'8 ~
            c'32 ]
            c'8
            c'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Leaf__set_duration_03():
    r'''Change leaf to nontied duration.
    Same as voice.written_duration = Duration(3, 16).
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8
            f'8
        }
        '''
        )

    voice[1]._set_duration(Duration(3, 16))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8. ]
            e'8
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Leaf__set_duration_04():
    r'''Change leaf to tied duration without power-of-two denominator.
    Tuplet inserted over new tied notes.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8
            f'8
        }
        '''
        )

    voice[1]._set_duration(Duration(5, 48))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                d'8 ~
                d'32 ]
            }
            e'8
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Leaf__set_duration_05():
    r'''Change leaf to untied duration without power-of-two denominator.
    Tuplet inserted over input leaf.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
            e'8
            f'8
        }
        '''
        )

    voice[1]._set_duration(Duration(1, 12))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                d'8 ]
            }
            e'8
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Leaf__set_duration_06():
    r'''Change leaf with LilyPond multiplier to untied duration with
    power-of-two denominator. LilyPond multiplier changes but leaf written
    duration does not.
    '''

    note = Note(0, (1, 8))
    attach(Multiplier(1, 2), note)

    assert format(note) == "c'8 * 1/2"

    note._set_duration(Duration(1, 32))

    assert inspect_(note).is_well_formed()
    assert format(note) == "c'8 * 1/4"


def test_scoretools_Leaf__set_duration_07():
    r'''Change leaf with LilyPond multiplier to untied duration with
    power-of-two denominator. LilyPond multiplier changes but leaf
    written duration does not.
    '''

    note = Note(0, (1, 8))
    attach(Multiplier(1, 2), note)

    assert format(note) == "c'8 * 1/2"

    note._set_duration(Duration(3, 32))

    assert inspect_(note).is_well_formed()
    assert format(note) == "c'8 * 3/4"


def test_scoretools_Leaf__set_duration_08():
    r'''Change leaf with LilyPond multiplier to tied duration with
    power-of-two denominator. LilyPond multiplier changes but leaf
    written duration does not.
    '''

    note = Note(0, (1, 8))
    attach(Multiplier(1, 2), note)

    assert format(note) == "c'8 * 1/2"

    note._set_duration(Duration(5, 32))

    assert inspect_(note).is_well_formed()
    assert format(note) == "c'8 * 5/4"


def test_scoretools_Leaf__set_duration_09():
    r'''Change leaf with LilyPond multiplier to duration without
    power-of-two denominator. LilyPond multiplier changes but leaf
    written duration does not.
    '''

    note = Note(0, (1, 8))
    attach(Multiplier(1, 2), note)

    assert format(note) == "c'8 * 1/2"

    note._set_duration(Duration(1, 24))

    assert inspect_(note).is_well_formed()
    assert format(note) == "c'8 * 1/3"


def test_scoretools_Leaf__set_duration_10():
    r'''Change leaf with LilyPond multiplier.
    Change to tie-necessitating duration without power-of-two denominator.
    LilyPond multiplier changes but leaf written duration does not.
    '''

    note = Note(0, (1, 8))
    attach(Multiplier(1, 2), note)

    assert format(note) == "c'8 * 1/2"

    note._set_duration(Duration(5, 24))

    assert inspect_(note).is_well_formed()
    assert format(note) == "c'8 * 5/3"


def test_scoretools_Leaf__set_duration_11():
    r'''Change rest duration.
    '''

    voice = Voice("c'8 r8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:3])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            r8
            e'8 ]
            f'8
        }
        '''
        )

    voice[1]._set_duration(Duration(5, 32))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            r8
            r32
            e'8 ]
            f'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_Accidental___init___01():
    r'''Accidentals can initialize from a string.
    '''

    accidental = pitchtools.Accidental('s')
    assert accidental == pitchtools.Accidental('s')


def test_pitchtools_Accidental___init___02():
    r'''Accidentals can initialize from other accidentals.
    '''

    accidental = pitchtools.Accidental(pitchtools.Accidental('s'))
    assert accidental == pitchtools.Accidental('s')


def test_pitchtools_Accidental___init___03():
    r'''Accidentals can initialize with none.
    '''

    accidental = pitchtools.Accidental(None)
    assert accidental == pitchtools.Accidental('')


def test_pitchtools_Accidental___init___04():
    r'''Accidentals can initialize with full accidental name.
    '''

    assert pitchtools.Accidental('sharp') == pitchtools.Accidental('s')
    assert pitchtools.Accidental('flat') == pitchtools.Accidental('f')
    assert pitchtools.Accidental('natural') == pitchtools.Accidental('')


def test_pitchtools_Accidental___init___05():

    assert pytest.raises(ValueError, "pitchtools.Accidental('foo')")


def test_pitchtools_Accidental___init___06():
    r'''Initialize with number.
    '''

    assert pitchtools.Accidental(0) == pitchtools.Accidental()
    assert pitchtools.Accidental(1) == pitchtools.Accidental('sharp')
    assert pitchtools.Accidental(-1) == pitchtools.Accidental('flat')
    assert pitchtools.Accidental(-5.5).semitones == -5.5
    assert pitchtools.Accidental(10).semitones == 10


def test_pitchtools_Accidental___init___07():
    r'''Initialize with symbolic string.
    '''

    assert pitchtools.Accidental('##') == pitchtools.Accidental('double sharp')
    assert pitchtools.Accidental('#') == pitchtools.Accidental('sharp')
    assert pitchtools.Accidental('b') == pitchtools.Accidental('flat')
    assert pitchtools.Accidental('bb') == pitchtools.Accidental('double flat')



# -*- coding: utf -*-
from abjad import *


def test_indicatortools_Tempo__to_markup_01():

    tempo = Tempo(Duration(1, 4), 60)
    markup = tempo._to_markup()

    assert format(markup) == stringtools.normalize(
        r'''
        \markup {
            \fontsize
                #-6
                \general-align
                    #Y
                    #DOWN
                    \note-by-number
                        #2
                        #0
                        #1
            \upright
                {
                    =
                    60
                }
            }
        '''
        ), format(markup)


def test_indicatortools_Tempo__to_markup_02():

    tempo = Tempo(Duration(3, 8), 60)
    markup = tempo._to_markup()

    assert format(markup) == stringtools.normalize(
        r'''
        \markup {
            \fontsize
                #-6
                \general-align
                    #Y
                    #DOWN
                    \note-by-number
                        #3
                        #1
                        #1
            \upright
                {
                    =
                    60
                }
            }
        '''
        ), format(markup)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_spannertools_PianoPedalSpanner_01():

    staff = Staff(scoretools.make_repeated_notes(4))
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sustainOn
            c'8
            c'8
            c'8 \sustainOff
        }
        '''
        )

    assert inspect_(staff).is_well_formed()
    assert piano_pedal_spanner.kind == 'sustain'
    assert piano_pedal_spanner.style == 'mixed'


def test_spannertools_PianoPedalSpanner_02():
    r'''Piano pedal spanner supports sostenuto pedal.
    '''

    staff = Staff("c'8 c'8 c'8 c'8")
    piano_pedal_spanner = spannertools.PianoPedalSpanner(kind='sostenuto')
    attach(piano_pedal_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sostenutoOn
            c'8
            c'8
            c'8 \sostenutoOff
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_03():
    r'''Piano pedal spanner supports una corda pedal.
    '''

    staff = Staff("c'8 c'8 c'8 c'8")
    piano_pedal_spanner = spannertools.PianoPedalSpanner(kind='corda')
    attach(piano_pedal_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \unaCorda
            c'8
            c'8
            c'8 \treCorde
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_04():
    r'''PianoPedal spanner supports text style.
    '''

    staff = Staff("c'8 c'8 c'8 c'8")
    piano_pedal_spanner = spannertools.PianoPedalSpanner(
        kind='sustain',
        style='text',
        )
    attach(piano_pedal_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'text
            c'8 \sustainOn
            c'8
            c'8
            c'8 \sustainOff
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_05():
    r'''PianoPedal spanner supports bracket style.
    '''

    staff = Staff("c'8 c'8 c'8 c'8")
    piano_pedal_spanner = spannertools.PianoPedalSpanner(
        kind='sustain',
        style='bracket',
        )
    attach(piano_pedal_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'bracket
            c'8 \sustainOn
            c'8
            c'8
            c'8 \sustainOff
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_06():
    r'''Consecutive dovetailing PianoPedal spanners format correctly.
    '''

    staff = Staff("c'8 c'8 c'8 c'8 c'8 c'8 c'8 c'8")
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[:4])
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[3:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sustainOn
            c'8
            c'8
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sustainOff \sustainOn
            c'8
            c'8
            c'8
            c'8 \sustainOff
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Component__has_spanner_01():

    staff = Staff("c'8 [ d'8 e'8 f'8 ]")
    assert staff[0]._has_spanner(Beam)


def test_scoretools_Component__has_spanner_02():

    assert not Note("c'8")._has_spanner(Beam)



# -*- coding: utf-8 -*-
from abjad.tools import pitchtools
from abjad.tools import tonalanalysistools


def create_pitch_contour_reservoir():
    r'''Creates pitch contour reservoir.
    '''

    scale = tonalanalysistools.Scale('a', 'minor')
    pitch_ranges = {
        'First Violin': pitchtools.PitchRange('[C4, A6]'),
        'Second Violin': pitchtools.PitchRange('[A3, A5]'),
        'Viola': pitchtools.PitchRange('[E3, A4]'),
        'Cello': pitchtools.PitchRange('[A2, A3]'),
        'Bass': pitchtools.PitchRange('[C3, A3]'),
    }

    reservoir = {}
    for instrument_name, pitch_range in pitch_ranges.items():
        pitch_set = scale.create_named_pitch_set_in_pitch_range(pitch_range)
        pitches = sorted(pitch_set, reverse=True)
        pitch_descents = []
        for i in range(len(pitches)):
            descent = tuple(pitches[:i + 1])
            pitch_descents.append(descent)
        reservoir[instrument_name] = tuple(pitch_descents)

    return reservoir


# -*- coding: utf-8 -*-
import numbers
from abjad.tools import durationtools
from abjad.tools import selectiontools


def make_rests(
    durations,
    decrease_durations_monotonically=True,
    tie_parts=False,
    ):
    r'''Make rests.

    Make rests and drecrease durations monotonically:

    ::

        >>> scoretools.make_rests(
        ...     [(5, 16), (9, 16)],
        ...     decrease_durations_monotonically=True,
        ...     )
        Selection([Rest('r4'), Rest('r16'), Rest('r2'), Rest('r16')])

    Makes rests and increase durations monotonically:

    ::

        >>> scoretools.make_rests(
        ...     [(5, 16), (9, 16)],
        ...     decrease_durations_monotonically=False,
        ...     )
        Selection([Rest('r16'), Rest('r4'), Rest('r16'), Rest('r2')])

    Make tied rests:

    ::

        >>> voice = Voice(scoretools.make_rests(
        ...     [(5, 16), (9, 16)],
        ...     tie_parts=True,
        ...     ))

    ..  doctest::

        >>> print(format(voice))
        \new Voice {
            r4
            r16
            r2
            r16
        }

    ::

        >>> show(voice) # doctest: +SKIP

    Returns list of rests.
    '''
    from abjad.tools import scoretools

    # check input
    if isinstance(durations, (numbers.Number, tuple)):
        durations = [durations]

    # make rests
    result = []
    for duration in durations:
        rests = scoretools.make_tied_leaf(
            scoretools.Rest,
            duration,
            pitches=None,
            decrease_durations_monotonically=decrease_durations_monotonically,
            tie_parts=tie_parts,
            )
        result.extend(rests)

    # return result
    result = selectiontools.Selection(result)
    return result



# -*- coding: utf-8 -*-
from abjad.tools import lilypondnametools
from abjad.tools import schemetools
from abjad.tools.abctools import AbjadValueObject


class LineSegment(AbjadValueObject):
    r'''Line segment.

    Line segments format as text spanners.

    ..  container:: example

        **Example 1.** Default line segment:

        ::

            >>> line_segment = indicatortools.LineSegment()
            >>> f(line_segment)
            LineSegment()

    .. todo:: Add examples.

    Use line segments to start a markup-terminated text spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_arrow_width',
        '_dash_fraction',
        '_dash_period',
        '_default_scope',
        '_left_broken_padding',
        '_left_broken_text',
        '_left_hspace',
        '_left_padding',
        '_left_stencil_align_direction_y',
        '_right_arrow',
        '_right_broken_arrow',
        '_right_broken_padding',
        '_right_padding',
        '_right_stencil_align_direction_y',
        '_style',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        arrow_width=None,
        dash_fraction=None,
        dash_period=None,
        left_broken_padding=None,
        left_broken_text=None,
        left_hspace=None,
        left_padding=None,
        left_stencil_align_direction_y=None,
        right_arrow=None,
        right_broken_arrow=None,
        right_broken_padding=None,
        right_padding=None,
        right_stencil_align_direction_y=None,
        style=None,
        ):
        self._default_scope = None
        self._arrow_width = arrow_width
        self._dash_fraction = dash_fraction
        self._dash_period = dash_period
        self._left_broken_padding = left_broken_padding
        self._left_broken_text = left_broken_text
        self._left_padding = left_padding
        self._left_hspace = left_hspace
        self._left_stencil_align_direction_y = left_stencil_align_direction_y
        self._right_arrow = right_arrow
        self._right_broken_arrow = right_broken_arrow
        self._right_broken_padding = right_broken_padding
        self._right_padding = right_padding
        self._right_stencil_align_direction_y = right_stencil_align_direction_y
        self._style = style

    ### PRIVATE METHODS ###

    def _get_lilypond_grob_overrides(self):
        overrides = []
        if self.arrow_width is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'arrow-width',
                    ),
                value=self.arrow_width,
                )
            overrides.append(override_)
        if self.dash_fraction is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'dash-fraction',
                    ),
                value=self.dash_fraction,
                )
            overrides.append(override_)
        if self.dash_period is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'dash-period',
                    ),
                value=self.dash_period,
                )
            overrides.append(override_)
        if self.left_broken_padding is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'left-broken',
                    'padding',
                    ),
                value=self.left_broken_padding,
                )
            overrides.append(override_)
        if self.left_broken_text is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'left-broken',
                    'text',
                    ),
                value=self.left_broken_text,
                )
            overrides.append(override_)
        if self.left_padding is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'left',
                    'padding',
                    ),
                value=self.left_padding,
                )
            overrides.append(override_)
        if self.left_stencil_align_direction_y is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'left',
                    'stencil-align-dir-y',
                    ),
                value=self.left_stencil_align_direction_y,
                )
            overrides.append(override_)
        if self.right_arrow is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'right',
                    'arrow',
                    ),
                value=self.right_arrow,
                )
            overrides.append(override_)
        if self.right_broken_arrow is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'right-broken',
                    'arrow',
                    ),
                value=self.right_broken_arrow,
                )
            overrides.append(override_)
        if self.right_broken_padding is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'right-broken',
                    'padding',
                    ),
                value=self.right_broken_padding,
                )
            overrides.append(override_)
        if self.right_padding is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'right',
                    'padding',
                    ),
                value=self.right_padding,
                )
            overrides.append(override_)
        if self.right_stencil_align_direction_y is not None:
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'bound-details',
                    'right',
                    'stencil-align-dir-y',
                    ),
                value=self.right_stencil_align_direction_y,
                )
            overrides.append(override_)
        if self.style is not None:
            style = schemetools.Scheme(self.style, quoting="'")
            override_ = lilypondnametools.LilyPondGrobOverride(
                grob_name='TextSpanner',
                is_once=True,
                property_path=(
                    'style',
                    ),
                value=style,
                )
            overrides.append(override_)
        return overrides

    ### PUBLIC PROPERTIES ###

    @property
    def arrow_width(self):
        r'''Gets arrow width of line segment.

        Returns float or none.
        '''
        return self._arrow_width

    @property
    def dash_fraction(self):
        r'''Gets dash fraction of line segment.

        Returns float or none.
        '''
        return self._dash_fraction

    @property
    def dash_period(self):
        r'''Gets dash period of line segment.

        Returns float or none.
        '''
        return self._dash_period

    @property
    def default_scope(self):
        r'''Gets default scope of line segment.

        Returns none.
        '''
        return self._default_scope

    @property
    def left_broken_padding(self):
        r'''Gets left broken padding of line segment.

        Returns float or none.
        '''
        return self._left_broken_padding

    @property
    def left_broken_text(self):
        r'''Gets left broken text of line segment.

        Returns markup, false or none.
        '''
        return self._left_broken_text

    @property
    def left_hspace(self):
        r'''Gets left hspace of line segment.

        Returns float or none.
        '''
        return self._left_hspace

    @property
    def left_padding(self):
        r'''Gets left padding of line segment.

        Returns float or none.
        '''
        return self._left_padding

    @property
    def left_stencil_align_direction_y(self):
        r'''Gets left stencil align direction Y of line segment.

        Returns float or none.
        '''
        return self._left_stencil_align_direction_y

    @property
    def right_arrow(self):
        r'''Is true when right end of line segment carries an arrow.
        Otherwise false.

        Returns true, false or none.
        '''
        return self._right_arrow

    @property
    def right_broken_arrow(self):
        r'''Gets right broken arrow of line segment.

        Returns float or none.
        '''
        return self._right_broken_arrow
        
    @property
    def right_broken_padding(self):
        r'''Gets right broken padding of line segment.

        Returns float or none.
        '''
        return self._right_broken_padding

    @property
    def right_padding(self):
        r'''Gets right padding of line segment.

        Returns float or none.
        '''
        return self._right_padding

    @property
    def right_stencil_align_direction_y(self):
        r'''Gets right stencil align direction Y of line segment.

        Returns float or none.
        '''
        return self._right_stencil_align_direction_y

    @property
    def style(self):
        r'''Gets style of line segment.

        Returns string or none.
        '''
        return self._style



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_RootlessChordClass___init___01():
    r'''Initialize root position triad.
    '''

    cqi = tonalanalysistools.RootlessChordClass('major', 'triad')
    assert str(cqi) == '<P1, +M3, +P5>'

    cqi = tonalanalysistools.RootlessChordClass('minor', 'triad')
    assert str(cqi) == '<P1, +m3, +P5>'

    cqi = tonalanalysistools.RootlessChordClass('diminished', 'triad')
    assert str(cqi) == '<P1, +m3, +dim5>'

    cqi = tonalanalysistools.RootlessChordClass('augmented', 'triad')
    assert str(cqi) == '<P1, +M3, +aug5>'


def test_tonalanalysistools_RootlessChordClass___init___02():
    r'''Initialize seventh and ninth.
    '''

    cqi = tonalanalysistools.RootlessChordClass('dominant', 7, 'root')
    assert str(cqi) == '<P1, +M3, +P5, +m7>'

    cqi = tonalanalysistools.RootlessChordClass('dominant', 9, 'root')
    assert str(cqi) == '<P1, +M3, +P5, +m7, +M9>'


def test_tonalanalysistools_RootlessChordClass___init___03():
    r'''Initialize with quality string and integer cardinality.
    '''

    cqi = tonalanalysistools.RootlessChordClass('dominant', 7)
    assert str(cqi) == '<P1, +M3, +P5, +m7>'

    cqi = tonalanalysistools.RootlessChordClass('major', 7)
    assert str(cqi) == '<P1, +M3, +P5, +M7>'

    cqi = tonalanalysistools.RootlessChordClass('diminished', 7)
    assert str(cqi) == '<P1, +m3, +dim5, +dim7>'

    cqi = tonalanalysistools.RootlessChordClass('dominant', 9)
    assert str(cqi) == '<P1, +M3, +P5, +m7, +M9>'



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__functions__language_01():

    target = Container([
        Note("cs'8"),
        Note("ds'8"),
        Note("ff'8")
    ])

    assert format(target) == stringtools.normalize(
        r'''
        {
            cs'8
            ds'8
            ff'8
        }
        '''
        )

    string = r"\language nederlands { cis'8 dis'8 fes'8 }"
    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools import TreeNode


class GraphvizField(TreeNode):
    r'''A Graphviz struct field.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = (
        '_edges',
        '_label',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        label=None,
        name=None,
        ):
        TreeNode.__init__(self, name=name)
        self._label = label
        self._edges = set([])

    ### PRIVATE PROPERTIES ###

    @property
    def _struct_format_contributions(self):
        result = '<{}>'.format(self.field_name)
        if self.label:
            result = '{} {}'.format(result, self.label)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def canonical_name(self):
        r'''Gets field canonical name.
        '''
        canonical_name = '{}:{}'.format(
            self.struct.canonical_name,
            self.field_name,
            )
        return canonical_name

    @property
    def edges(self):
        r'''Gets edges of Graphviz struct field.
        '''
        return tuple(self._edges)

    @property
    def field_name(self):
        r'''Gets the field name.
        '''
        graph_order = self.graph_order
        struct = self.struct
        if struct is not None:
            struct_graph_order = struct.graph_order
            graph_order = graph_order[len(struct_graph_order):]
        return 'f_' + '_'.join(str(x) for x in graph_order)

    @property
    def label(self):
        r'''Gets the field label.
        '''
        return self._label

    @property
    def struct(self):
        r'''Gets the parent struct.
        '''
        from abjad.tools import graphtools
        parent = self.parent
        while parent is not None:
            if isinstance(parent, graphtools.GraphvizNode):
                return parent
            parent = parent.parent



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSet___contains___01():
    r'''Pitch set containment works as expected.
    '''

    pitch_set = pitchtools.PitchSet([12, 14, 18, 19])

    assert NamedPitch(14) in pitch_set
    assert NamedPitch(15) not in pitch_set



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_repeated_rests_from_time_signatures_01():
    r'''Make repeated rests from list of integer pairs.
    '''

    rests = scoretools.make_repeated_rests_from_time_signatures(
        [(2, 8), (3, 32)])
    assert len(rests) == 2

    rests = Sequence(rests).flatten()
    staff = Staff(rests)

    r'''
    \new Staff {
        r8
        r8
        r32
        r32
        r32
    }
    '''

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            r8
            r8
            r32
            r32
            r32
        }
        '''
        )
    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Tuplet_timespan_01():

    staff = Staff(r"c'4 d'4 \times 2/3 { e'4 f'4 g'4 }")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4
            d'4
            \times 2/3 {
                e'4
                f'4
                g'4
            }
        }
        '''
        )

    assert inspect_(staff).get_timespan() == \
        timespantools.Timespan(0, 1)
    assert inspect_(staff[0]).get_timespan() == \
        timespantools.Timespan(0, (1, 4))
    assert inspect_(staff[1]).get_timespan() == \
        timespantools.Timespan((1, 4), (1, 2))
    assert inspect_(staff[-1]).get_timespan() == \
        timespantools.Timespan((1, 2), 1)



# -*- coding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class LilyPondCommand(AbjadValueObject):
    r'''LilyPond command.

    ..  container:: example

        **Example 1.** Dotted slur:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> slur = spannertools.Slur()
            >>> attach(slur, staff[:])
            >>> command = indicatortools.LilyPondCommand('slurDotted')
            >>> attach(command, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \slurDotted
                c'8 (
                d'8
                e'8
                f'8 )
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_format_slot',
        '_name',
        '_prefix',
        )

    _format_leaf_children = False

    _allowable_format_slots = (
        'after',
        'before',
        'closing',
        'opening',
        'right',
        )

    ### INITIALIZER ###

    def __init__(self, name=None, format_slot=None, prefix='\\'):
        self._default_scope = None
        name = name or 'slurDotted'
        format_slot = format_slot or 'opening'
        assert format_slot in self._allowable_format_slots, repr(format_slot)
        assert isinstance(name, str), repr(name)
        self._name = name
        self._format_slot = format_slot
        assert isinstance(prefix, str), repr(prefix)
        self._prefix = prefix

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LilyPond command.

        Set `format_specification` to `''`, `'lilypond`' or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        elif format_specification == 'lilypond':
            return self._get_lilypond_format()
        return str(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self.name)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        names = []
        if not self.format_slot == 'opening':
            names.append('format_slot')
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=[self.name],
            storage_format_kwargs_names=names,
            storage_format_is_indented=False,
            )

    def _get_lilypond_format(self):
        command = self._name
        if command.startswith('#'):
            return command
        elif ' ' not in command:
            return self.prefix + stringtools.to_lower_camel_case(command)
        else:
            return self.prefix + command

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        format_slot = lilypond_format_bundle.get(self.format_slot)
        format_slot.commands.append(self._get_lilypond_format())
        return lilypond_format_bundle

    ### PUBLIC METHODS ###

    @staticmethod
    def list_allowable_format_slots():
        r'''Lists allowable format slots.

        ..  container:: example

            **Example 1.** Default:

                >>> commands = indicatortools.LilyPondCommand.list_allowable_format_slots()
                >>> for command in commands:
                ...     command
                'after'
                'before'
                'closing'
                'opening'
                'right'

        Returns tuple of strings.
        '''
        return LilyPondCommand._allowable_format_slots

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of LilyPond command.

        ..  container:: example

            **Example 1.** Dotted slur:

            ::

                >>> command = indicatortools.LilyPondCommand('slurDotted')
                >>> command.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def format_slot(self):
        r'''Gets format slot of LilyPond command.

        ..  container:: example

            **Example 1.** Dotted slur:

            ::

                >>> command = indicatortools.LilyPondCommand('slurDotted')
                >>> command.format_slot
                'opening'

        Defaults to `'opening'`.

        Returns string.
        '''
        return self._format_slot

    @property
    def name(self):
        r'''Gets name of LilyPond command.

        ..  container:: example

            **Example 1.** Dotted slur:

            ::

                >>> command = indicatortools.LilyPondCommand('slurDotted')
                >>> command.name
                'slurDotted'

        Returns string.
        '''
        return self._name

    @property
    def prefix(self):
        r'''Gets prefix.

        Defaults to `'\\'`.

        Set to string.

        Returns string.
        '''
        return self._prefix



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch_pitch_class_octave_label_01():

    named_pitch = NamedPitch("cs''")
    assert named_pitch.pitch_class_octave_label == 'C#5'



# -*- coding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.lilypondnametools.LilyPondNameManager \
    import LilyPondNameManager


class LilyPondGrobNameManager(LilyPondNameManager):
    '''LilyPond grob name manager.

    ..  container:: example

        Initializes with toplevel override function:

        ::

            >>> note = Note("c'4")
            >>> override(note)
            LilyPondGrobNameManager()

    '''

    ### SPECIAL METHODS ###

    def __getattr__(self, name):
        r'''Gets LilyPond name manager keyed to `name`.

        ..  container:: example

            ::

                >>> staff = Staff("c'4 d' e' f'")
                >>> override(staff).note_head.color = 'red'
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override NoteHead.color = #red
                } {
                    c'4
                    d'4
                    e'4
                    f'4
                }

        ..  container:: example

            Returns LilyPond name manager:

            ::

                >>> override(staff).note_head
                LilyPondNameManager(('color', 'red'))

        '''
        from abjad import ly
        from abjad.tools import lilypondnametools
        camel_name = stringtools.to_upper_camel_case(name)
        if name.startswith('_'):
            try:
                return vars(self)[name]
            except KeyError:
                message = '{!r} object has no attribute: {!r}.'
                message = message.format(type(self).__name__, name)
                raise AttributeError(message)
        elif camel_name in ly.contexts:
            try:
                return vars(self)['_' + name]
            except KeyError:
                context = lilypondnametools.LilyPondGrobNameManager()
                vars(self)['_' + name] = context
                return context
        elif camel_name in ly.grob_interfaces:
            try:
                return vars(self)[name]
            except KeyError:
                vars(self)[name] = lilypondnametools.LilyPondNameManager()
                return vars(self)[name]
        else:
            try:
                return vars(self)[name]
            except KeyError:
                message = '{!r} object has no attribute: {!r}.'
                message = message.format(type(self).__name__, name)
                raise AttributeError(message)

    def __setattr__(self, attribute_name, value):
        r'''Sets attribute `attribute_name` of grob name manager to `value`.

        Returns none.
        '''
        # make sure attribute name is valid grob name before setting value
        attribute_value = getattr(self, attribute_name)
        object.__setattr__(self, attribute_name, value)

    ### PRIVATE METHODS ###

    def _get_attribute_tuples(self):
        from abjad.tools import lilypondnametools
        result = []
        for name, value in vars(self).items():
            if type(value) is lilypondnametools.LilyPondNameManager:
                grob_name, grob_proxy = name, value
                pairs = iter(vars(grob_proxy).items())
                for attribute_name, attribute_value in pairs:
                    triple = (grob_name, attribute_name, attribute_value)
                    result.append(triple)
            else:
                context_name, context_proxy = name.strip('_'), value
                for grob_name, grob_proxy in vars(context_proxy).items():
                    pairs = iter(vars(grob_proxy).items())
                    for attribute_name, attribute_value in pairs:
                        quadruple = (
                            context_name,
                            grob_name,
                            attribute_name,
                            attribute_value,
                            )
                        result.append(quadruple)
        return tuple(result)

    def _list_format_contributions(self, contribution_type, is_once=False):
        from abjad.tools import systemtools
        manager = systemtools.LilyPondFormatManager
        assert contribution_type in ('override', 'revert')
        result = []
        for attribute_tuple in self._get_attribute_tuples():
            if len(attribute_tuple) == 3:
                context_name = None
                grob_name = attribute_tuple[0]
                attribute_name = attribute_tuple[1]
                attribute_value = attribute_tuple[2]
            elif len(attribute_tuple) == 4:
                context_name = attribute_tuple[0]
                grob_name = attribute_tuple[1]
                attribute_name = attribute_tuple[2]
                attribute_value = attribute_tuple[3]
            else:
                raise ValueError
            if contribution_type == 'override':
                override_string = manager.make_lilypond_override_string(
                    grob_name,
                    attribute_name,
                    attribute_value,
                    context_name=context_name,
                    is_once=is_once,
                    )
                result.append(override_string)
            else:
                revert_string = manager.make_lilypond_revert_string(
                    grob_name,
                    attribute_name,
                    context_name=context_name,
                    )
                result.append(revert_string)
        result.sort()
        return result

    def _make_override_dictionary(self):
        result = {}
        grob_override_tuples = self._get_attribute_tuples()
        for grob_override_tuple in grob_override_tuples:
            most = '__'.join(grob_override_tuple[:-1])
            value = grob_override_tuple[-1]
            attribute = '_tools_package_qualified_repr'
            value = getattr(value, attribute, repr(value))
            result[most] = value
        return result



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Clef___repr___01():
    r'''Clef returns a nonempty repr string.
    '''

    repr = Clef('treble').__repr__()
    assert isinstance(repr, str) and 0 < len(repr)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Tempo___mul___01():

    tempo = Tempo(Duration(1, 4), 60)
    result = tempo * Duration(1, 2)
    assert result == Tempo(Duration(1, 4), 30)

    result = tempo * Duration(2, 3)
    assert result == Tempo(Duration(1, 4), 40)

    result = tempo * Duration(3, 4)
    assert result == Tempo(Duration(1, 4), 45)

    result = tempo * Duration(4, 5)
    assert result == Tempo(Duration(1, 4), 48)

    result = tempo * Duration(5, 6)
    assert result == Tempo(Duration(1, 4), 50)

    result = tempo * Duration(6, 7)
    assert result == Tempo(
        Duration(1, 4), Duration(360, 7))


def test_indicatortools_Tempo___mul___02():

    tempo = Tempo(Duration(1, 4), 60)
    result = tempo * 1
    assert result == Tempo(Duration(1, 4), 60)

    result = tempo * 2
    assert result == Tempo(Duration(1, 4), 120)

    result = tempo * 3
    assert result == Tempo(Duration(1, 4), 180)

    result = tempo * 4
    assert result == Tempo(Duration(1, 4), 240)


def test_indicatortools_Tempo___mul___03():

    tempo_1 = Tempo(textual_indication='Langsam')
    pytest.raises(ImpreciseTempoError, "tempo_1 * Duration(1, 2)")


def test_indicatortools_Tempo___mul___04():

    tempo_1 = Tempo(Duration(1, 8), (90, 92))
    pytest.raises(ImpreciseTempoError, "tempo_1 * Duration(1, 2)")



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure___format___01():

    measure = Measure((4, 16), "c'8 c'16 c'16")
    
    assert format(measure, 'storage') == stringtools.normalize(
        r'''
        scoretools.Measure(
            indicatortools.TimeSignature((4, 16)),
            "c'8 c'16 c'16"
            )
        '''
        )


def test_scoretools_Measure___format___02():

    measure = Measure((4, 16), "c'8 c'16 c'16", implicit_scaling=True)
    
    assert format(measure, 'storage') == stringtools.normalize(
        r'''
        scoretools.Measure(
            indicatortools.TimeSignature((4, 16)),
            "c'8 c'16 c'16",
            implicit_scaling=True,
            )
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools import TreeContainer
from abjad.tools.graphtools.GraphvizMixin import GraphvizMixin


class GraphvizNode(GraphvizMixin, TreeContainer):
    r'''A Graphviz node.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = (
        '_attributes',
        '_edges',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        attributes=None,
        children=None,
        name=None,
        ):
        TreeContainer.__init__(
            self,
            children=children,
            name=name,
            )
        GraphvizMixin.__init__(
            self,
            attributes=attributes,
            )
        self._edges = set([])

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of Graphviz node.

        Returns string.
        '''
        result = self._graphviz_format_contributions
        result = '\n'.join(result)
        return result

    ### PUBLIC METHODS ###

    def attach(self, other):
        r'''Attaches node to attachable Graphviz object.

        ::

            >>> my_graph = graphtools.GraphvizGraph()
            >>> node_one = graphtools.GraphvizNode(attributes={'label': 'One'})
            >>> node_two = graphtools.GraphvizNode(attributes={'label': 'Two'})
            >>> my_graph.extend([node_one, node_two])
            >>> edge = node_one.attach(node_two)
            >>> graph(my_graph)  # doctest: +SKIP

        ..  doctest::

            >>> print(str(my_graph))
            digraph G {
                node_0 [label=One];
                node_1 [label=Two];
                node_0 -> node_1;
            }

        Returns GraphvizEdge.
        '''
        from abjad.tools import graphtools
        edge = graphtools.GraphvizEdge()
        edge.attach(self, other)
        return edge

    ### PRIVATE PROPERTIES ###

    @property
    def _graphviz_format_contributions(self):
        from abjad.tools import graphtools

        node_def = self._format_value(self.canonical_name)
        attributes = self.attributes

        if len(self) == 1 and not isinstance(self[0], (
            graphtools.GraphvizField,
            graphtools.GraphvizGroup,
            )):
            attributes['label'] = self[0]
        elif len(self):
            struct_format_contributions = self._struct_format_contributions
            if struct_format_contributions:
                attributes['label'] = struct_format_contributions

        if len(attributes):
            result = []
            result.extend(self._format_attribute_list(attributes))
            result[0] = '{} {}'.format(node_def, result[0])
            return result

        return [node_def + ';']

    @property
    def _node_class(self):
        from abjad.tools import graphtools
        prototype = (
            graphtools.GraphvizField,
            graphtools.GraphvizGroup,
            graphtools.GraphvizTable,
            )
        return prototype

    @property
    def _struct_format_contributions(self):
        result = []
        for x in self:
            part = x._struct_format_contributions
            if part:
                result.append(part)
        result = ' | '.join(result)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def all_edges(self):
        r'''Gets edges of this node and those of any field in its field
        subtree.
        '''
        from abjad.tools import graphtools
        edges = set(self.edges)
        prototype = (
            graphtools.GraphvizGroup,
            graphtools.GraphvizTable,
            graphtools.GraphvizTableCell,
            graphtools.GraphvizTableRow,
            graphtools.GraphvizTableHorizontalRule,
            graphtools.GraphvizTableVerticalRule,
            )
        for node in self.nodes[1:]:
            if isinstance(node, prototype):
                continue
            edges.update(node.edges)
        return tuple(edges)

    @property
    def canonical_name(self):
        r'''Canonical name of Graphviz node.

        Returns string.
        '''
        if self.name is not None:
            return self.name
        if self.graph_order:
            return 'node_' + '_'.join(str(x) for x in self.graph_order)
        return 'node_0'

    @property
    def edges(self):
        r'''Edges of Graphviz node.

        Returns tuple.
        '''
        return tuple(self._edges)



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools import TreeContainer


class GraphvizTable(TreeContainer):
    r'''A Graphviz table.

    ::

        >>> table = graphtools.GraphvizTable(
        ...     attributes={'style': 'rounded'},
        ...     )
        >>> row_1 = graphtools.GraphvizTableRow()
        >>> row_1.append(graphtools.GraphvizTableCell(label='foo'))
        >>> row_1.append(graphtools.GraphvizTableVerticalRule())
        >>> row_1.append(graphtools.GraphvizTableCell(label='bar'))
        >>> row_2 = graphtools.GraphvizTableRow()
        >>> row_2.append(graphtools.GraphvizTableCell(label='quux'))
        >>> table.extend([row_1, row_2])
        >>> print(table)
        <
        <TABLE STYLE="ROUNDED">
            <TR>
                <TD>foo</TD>
                <VR/>
                <TD>bar</TD>
            </TR>
            <TR>
                <TD>quux</TD>
            </TR>
        </TABLE>>

    ::

        >>> node = graphtools.GraphvizNode()
        >>> node.append(table)
        >>> print(node)
        node_0 [label=<
            <TABLE STYLE="ROUNDED">
                <TR>
                    <TD>foo</TD>
                    <VR/>
                    <TD>bar</TD>
                </TR>
                <TR>
                    <TD>quux</TD>
                </TR>
            </TABLE>>];

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = (
        '_attributes',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        children=None,
        name=None,
        attributes=None,
        ):
        TreeContainer.__init__(
            self,
            children=children,
            name=name,
            )
        if attributes is not None:
            attributes = dict(attributes)
        self._attributes = attributes

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Get string representation of Graphviz table.

        Returns string.
        '''
        result = ['<']
        attribute_string = self._attribute_string
        if attribute_string:
            result.append('<TABLE {}>'.format(attribute_string))
        else:
            result.append('<TABLE>')
        for x in self:
            for line in str(x).splitlines():
                result.append('    ' + line)
        result.append('</TABLE>>')
        result = '\n'.join(result)
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_string(self):
        if not self.attributes:
            return ''
        result = []
        for key, value in sorted(self.attributes.items()):
            attribute = '{}="{}"'.format(key.upper(), str(value).upper())
            result.append(attribute)
        result = ' '.join(result)
        return result

    @property
    def _node_class(self):
        from abjad.tools import graphtools
        prototype = (
            graphtools.GraphvizTableRow,
            graphtools.GraphvizTableHorizontalRule,
            )
        return prototype

    ### PUBLIC PROPERTIES ###

    @property
    def attributes(self):
        r'''Gets Graphviz table attribute dictionary.

        Returns dictionary.
        '''
        return self._attributes



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TypedOrderedDict___eq___01():

    dictionary_1 = datastructuretools.TypedOrderedDict([
        ('flavor', 'cherry'), ('count', 2),
        ])
    dictionary_2 = datastructuretools.TypedOrderedDict([
        ('flavor', 'cherry'), ('count', 2),
        ])
    dictionary_3 = datastructuretools.TypedOrderedDict([
        ('flavor', 'chocolate'), ('count', 2),
        ])

    assert dictionary_1 == dictionary_1
    assert dictionary_1 == dictionary_2
    assert not dictionary_1 == dictionary_3
    assert dictionary_2 == dictionary_1
    assert dictionary_2 == dictionary_2
    assert not dictionary_2 == dictionary_3
    assert not dictionary_3 == dictionary_1
    assert not dictionary_3 == dictionary_2
    assert dictionary_3 == dictionary_3


def test_datastructuretools_TypedOrderedDict___eq___02():

    dictionary_1 = datastructuretools.TypedOrderedDict([
        ('flavor', 'cherry'), ('count', 2),
        ])
    dictionary_2 = datastructuretools.TypedOrderedDict([
        ('flavor', 'cherry'), ('count', 2), ('color', 'red'),
        ])

    assert dictionary_1 == dictionary_1
    assert not dictionary_1 == dictionary_2
    assert not dictionary_2 == dictionary_1
    assert dictionary_2 == dictionary_2



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import iterate


class Tie(Spanner):
    r'''Tie.

    ..  container:: example

        Ties four notes:

        ::

            >>> staff = Staff("c'4 c' c' c'")
            >>> attach(Tie(), staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4 ~
                c'4 ~
                c'4 ~
                c'4
            }

    ..  container:: example

        Fails attachment test when pitches differ:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> attach(Tie(), staff[:])
            Traceback (most recent call last):
                ...
            Exception: Tie() attachment test fails for Selection([Note("c'4"), Note("d'4"), Note("e'4"), Note("f'4")]).

    ..  container:: example

        Ties consecutive chords if all adjacent pairs have at least one pitch in common:

        ::

            >>> staff = Staff("<c'>4 <c' d'>4 <d'>4")
            >>> attach(Tie(), staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                <c'>4 ~
                <c' d'>4 ~
                <d'>4
            }

    Formats LilyPond ``~`` command on nonlast leaves in spanner.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        '_use_messiaen_style_ties',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        direction=None,
        overrides=None,
        use_messiaen_style_ties=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        direction = stringtools.to_tridirectional_lilypond_symbol(
            direction)
        self._direction = direction
        self._use_messiaen_style_ties = use_messiaen_style_ties

    ### PRIVATE METHODS ###

    def _attachment_test(self, component):
        from abjad.tools import scoretools
        pitched_prototype = (scoretools.Note, scoretools.Chord)
        return isinstance(component, pitched_prototype)

    def _attachment_test_all(self, component_expression):
        from abjad.tools import scoretools
        from abjad.tools import sequencetools
        #if not self._at_least_two_leaves(component_expression):
        #    return False
        written_pitches = []
        if isinstance(component_expression, scoretools.Component):
            component_expression = [component_expression]
        for component in component_expression:
            if isinstance(component, scoretools.Note):
                written_pitches.append(set([component.written_pitch]))
            elif isinstance(component, scoretools.Chord):
                written_pitches.append(set(component.written_pitches))
            else:
                return False
        for pair in sequencetools.Sequence(written_pitches).nwise():
            if not set.intersection(*pair):
                return False
        return True

    def _copy_keyword_args(self, new):
        new._direction = self.direction
        new._use_messiaen_style_ties = self.use_messiaen_style_ties

    def _format_right_of_leaf(self, leaf):
        from abjad.tools import scoretools
        result = []
        prototype = (
            scoretools.Container,
            scoretools.Rest,
            scoretools.Skip,
            scoretools.MultimeasureRest,
            )
        if not self.use_messiaen_style_ties:
            if self._is_my_last_leaf(leaf):
                return result
            elif isinstance(leaf, prototype):
                return result
            elif isinstance(leaf._get_leaf(1), prototype):
                return result
            if self.direction is not None:
                string = '{} ~'.format(self.direction)
                result.append(string)
            else:
                result.append('~')
        else:
            if isinstance(leaf, prototype):
                return result
            elif self._is_my_first_leaf(leaf):
                return result
            if self.direction is not None:
                string = r'{} \repeatTie'.format(self.direction)
                result.append(string)
            else:
                result.append(r'\repeatTie')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets direction.

        ..  container:: example

            Forces ties up:

            ::

                >>> staff = Staff("c'8 c'8 c'8 c'8")
                >>> tie = Tie(direction=Up)
                >>> attach(tie, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 ^ ~
                    c'8 ^ ~
                    c'8 ^ ~
                    c'8
                }

            ::

                >>> tie.direction
                '^'

        ..  container:: example

            Forces ties down:

            ::

                >>> staff = Staff("c'8 c'8 c'8 c'8")
                >>> tie = Tie(direction=Down)
                >>> attach(tie, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 _ ~
                    c'8 _ ~
                    c'8 _ ~
                    c'8
                }

            ::

                >>> tie.direction
                '_'

        ..  container:: example

            Positions ties according to LilyPond defaults:

            ::

                >>> staff = Staff("c'8 c'8 c'8 c'8")
                >>> tie = Tie(direction=None)
                >>> attach(tie, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8 ~
                    c'8 ~
                    c'8 ~
                    c'8
                }

            ::

                >>> tie.direction is None
                True

        Defaults to none.

        Set to up, down or none.

        Returns up, down or none.
        '''
        return self._direction

    @property
    def use_messiaen_style_ties(self):
        r'''Is true when tie should use Messiaen-style ties with
        the LilyPond ``\repeatTie`` command.

        ..  container:: example

            Default values for Messiaen-style ties:

            ::

                >>> staff = Staff("c'8 c'8 c'8 c'8")
                >>> tie = Tie(direction=Up, use_messiaen_style_ties=True)
                >>> attach(tie, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'8
                    c'8 ^ \repeatTie
                    c'8 ^ \repeatTie
                    c'8 ^ \repeatTie
                }

            LilyPond's repeat ties are shorter Messiaen-style ties.

        Returns true, false or none.
        '''
        return self._use_messiaen_style_ties



# -*- coding: utf-8 -*-
import math


def binomial_coefficient(n, k):
    r'''Gets binomial coefficient of `n` choose `k`.

    ..  container:: example

        ::

            >>> for k in range(8):
            ...     print(k, '\t', mathtools.binomial_coefficient(8, k))
            ... 
            0  1
            1  8
            2  28
            3  56
            4  70
            5  56
            6  28
            7  8

    Returns positive integer.
    '''
    return math.factorial(n) // (math.factorial(n - k) * math.factorial(k))



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ScaleDegree_name_01():

    assert tonalanalysistools.ScaleDegree(1).name == 'tonic'
    assert tonalanalysistools.ScaleDegree(2).name == 'superdominant'
    assert tonalanalysistools.ScaleDegree(3).name == 'mediant'
    assert tonalanalysistools.ScaleDegree(4).name == 'subdominant'
    assert tonalanalysistools.ScaleDegree(5).name == 'dominant'
    assert tonalanalysistools.ScaleDegree(6).name == 'submediant'
    assert tonalanalysistools.ScaleDegree(7).name == 'leading tone'



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner___in___01():
    r'''Spanner containment tests components.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[1][:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                d'8
            }
            {
                e'8 [
                f'8 ]
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    assert voice[1] not in beam
    assert voice[1][0] in beam
    assert voice[1][1] in beam


# -*- coding: utf-8 -*-
import platform
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools


@unittest.skipIf(
    platform.python_implementation() != 'CPython',
    'Only for CPython.',
    )
class TestLaTeXDocumentHandler_syntax_error(unittest.TestCase):

    maxDiff = None

    def test_syntax_error_1(self):
        input_file_contents = [
            '<abjad>',
            'foo bar baz',
            '</abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            abjadbooktools.AbjadBookError,
            document_handler.__call__,
            )

    def test_syntax_error_2(self):
        input_file_contents = [
            '<abjad>[allow_exceptions=true]',
            'foo bar baz',
            '</abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        rebuilt_source = document_handler(return_source=True)
        assert rebuilt_source == stringtools.normalize(
            '''
            <abjad>[allow_exceptions=true]
            foo bar baz
            </abjad>

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            >>> foo bar baz
              File "<stdin>", line 1
                foo bar baz
                      ^
            SyntaxError: invalid syntax
            \\end{lstlisting}
            %%% ABJADBOOK END %%%
            ''',
            )

    def test_syntax_error_3(self):
        input_file_contents = [
            '<abjad>[allow_exceptions=true]',
            'foo bar baz',
            '</abjad>',
            '',
            '<abjad>',
            'foo bar baz',
            '</abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            abjadbooktools.AbjadBookError,
            document_handler.__call__
            )



# -*- coding: utf-8 -*-
import collections
from abjad.tools.abctools import AbjadValueObject
from abjad.tools import selectiontools
from abjad.tools.topleveltools import iterate


class RunSelectorCallback(AbjadValueObject):
    r'''Run selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_prototype',
        )

    ### INITIALIZER ###

    def __init__(self, prototype=None):
        prototype = prototype or ()
        if isinstance(prototype, collections.Sequence):
            prototype = tuple(prototype)
            assert all(isinstance(x, type) for x in prototype)
        assert isinstance(prototype, (tuple, type))
        self._prototype = prototype

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple of selections.
        '''
        assert isinstance(argument, tuple), repr(argument)
        result = []
        prototype = self.prototype
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        for subexpr in argument:
            for run in iterate(subexpr).by_run(prototype):
                run = selectiontools.Selection(run)
                result.append(run)
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def prototype(self):
        r'''Gets run selector callback prototype.

        Return tuple of classes.
        '''
        return self._prototype



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NamedPitch___int___01():
    r'''Returns pitch number of 12-ET named pitch as int.
    '''

    named_pitch = NamedPitch(13)
    assert isinstance(int(named_pitch), int)
    assert int(named_pitch) == 13


def test_pitchtools_NamedPitch___int___02():
    r'''Raise type error on non-12-ET named pitch.
    '''

    named_pitch = NamedPitch(13.5)
    assert pytest.raises(TypeError, 'int(named_pitch)')



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedPitch___add___01():
    r'''Add numbered pitch to numbered pitch.
    '''

    pitch_1 = pitchtools.NumberedPitch(12)
    pitch_2 = pitchtools.NumberedPitch(13)

    assert pitch_1 + pitch_2 == pitchtools.NumberedPitch(25)


def test_pitchtools_NumberedPitch___add___02():
    r'''Add number to numbered pitch.
    '''

    pitch_1 = pitchtools.NumberedPitch(12)

    assert pitch_1 + 13 == pitchtools.NumberedPitch(25)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitchClass_diatonic_pitch_class_name_01():

    assert pitchtools.NamedPitchClass('c').diatonic_pitch_class_name == 'c'
    assert pitchtools.NamedPitchClass('cs').diatonic_pitch_class_name == 'c'
    assert pitchtools.NamedPitchClass('cf').diatonic_pitch_class_name == 'c'
    assert pitchtools.NamedPitchClass('cqs').diatonic_pitch_class_name == 'c'
    assert pitchtools.NamedPitchClass('cqf').diatonic_pitch_class_name == 'c'



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


class ExtraLeafSelectorCallback(AbjadValueObject):
    r'''Extra leaf selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_with_next_leaf',
        '_with_previous_leaf',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        with_next_leaf=False,
        with_previous_leaf=False,
        ):
        self._with_next_leaf = bool(with_next_leaf)
        self._with_previous_leaf = bool(with_previous_leaf)

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple in which each item is a selection or component.
        '''
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        from abjad.tools.topleveltools import select
        assert isinstance(argument, tuple), repr(argument)
        result = []
        for subexpr in argument:
            if isinstance(subexpr, scoretools.Leaf):
                subexpr = selectiontools.Selection([subexpr])
            subresult = []
            if self.with_previous_leaf:
                if isinstance(subexpr[0], selectiontools.LogicalTie):
                    first_leaf = subexpr[0].head
                else:
                    first_leaf = subexpr[0]
                previous_leaf = first_leaf._get_leaf(-1)
                if previous_leaf is not None:
                    subresult.append(previous_leaf)
            if isinstance(subexpr, selectiontools.Selection):
                subresult.extend(subexpr)
            else:
                subresult.append(subexpr)
            if self.with_next_leaf:
                if isinstance(subexpr[-1], selectiontools.LogicalTie):
                    last_leaf = subexpr[-1].tail
                else:
                    last_leaf = subexpr[-1]
                next_leaf = last_leaf._get_leaf(1)
                if next_leaf is not None:
                    subresult.append(next_leaf)
            subresult = select(subresult)
            result.append(subresult)
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def with_next_leaf(self):
        r'''Gets next leaf inclusion.

        Returns true or false.
        '''
        return self._with_next_leaf

    @property
    def with_previous_leaf(self):
        r'''Gets previous leaf inclusion.

        Returns true or false.
        '''
        return self._with_previous_leaf
